// This is a TextMate grammar distributed by `starry-night`.
// This grammar is licensed `isc`.
// See <https://github.com/wooorm/starry-night> for more info.
/** @type {import('../lib/index.js').Grammar} */
const grammar = {
  extensions: ['.ms', '.mcr'],
  names: ['maxscript'],
  patterns: [{include: '#main'}],
  repository: {
    arrays: {
      patterns: [
        {
          begin: '#\\s*\\(',
          beginCaptures: {
            0: {name: 'punctuation.definition.array.begin.maxscript'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.definition.array.end.maxscript'}
          },
          name: 'storage.type.array.maxscript',
          patterns: [{include: '#main'}]
        },
        {
          begin: '(#)\\s*\\{',
          beginCaptures: {
            0: {name: 'punctuation.definition.bit.array.begin.maxscript'}
          },
          end: '\\}',
          endCaptures: {
            0: {name: 'punctuation.definition.bit.array.end.maxscript'}
          },
          name: 'storage.type.bitarray.maxscript',
          patterns: [
            {
              captures: {
                1: {name: 'bit-range.begin'},
                2: {name: 'bit-range.middle'},
                3: {name: 'bit-range.end'}
              },
              match: '(?<!\\$)\\b(\\d+)(\\.{2})(\\d+)\\b',
              name: 'storage.type.bit-range.maxscript'
            },
            {include: '#main'}
          ]
        }
      ]
    },
    comments: {
      patterns: [
        {
          begin: '--',
          beginCaptures: {
            0: {name: 'punctuation.definition.comment.maxscript'}
          },
          end: '$',
          name: 'comment.line.double-dash.maxscript'
        },
        {
          begin: '/\\*',
          beginCaptures: {
            0: {name: 'punctuation.definition.comment.begin.maxscript'}
          },
          end: '\\*/',
          endCaptures: {
            0: {name: 'punctuation.definition.comment.end.maxscript'}
          },
          name: 'comment.block.maxscript'
        }
      ]
    },
    constants: {
      patterns: [
        {
          match: '(?i)\\b(?<![\\.\\$])(true|on)(?!\\s*:)\\b',
          name: 'constant.language.boolean.true.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])(false|off)(?!\\s*:)\\b',
          name: 'constant.language.boolean.false.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])pi(?!\\s*:)\\b',
          name: 'constant.language.math.pi.maxscript'
        },
        {
          match:
            '(?i)\\b(?<![\\.\\$])(red|green|blue|white|black|orange|yellow|brown|gray)(?!\\s*:)\\b',
          name: 'constant.language.colour.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])[xyz]_axis(?!\\s*:)\\b',
          name: 'constant.language.axis.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])ok(?!\\s*:)\\b',
          name: 'constant.language.ok.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])undefined(?!\\s*:)\\b',
          name: 'constant.language.undefined.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])unsupplied(?!\\s*:)\\b',
          name: 'constant.language.unsupplied.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])dontcollect(?!\\s*:)\\b',
          name: 'constant.language.dontcollect.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])e(?!\\s*:)\\b',
          name: 'constant.language.math.euler.maxscript'
        }
      ]
    },
    float: {
      patterns: [
        {
          captures: {
            1: {name: 'leading.decimal'},
            2: {name: 'decimal.separator'},
            3: {name: 'trailing.decimal'},
            4: {name: 'exponential.decimal'}
          },
          match: '(?<!\\$)\\b(\\d+)(\\.)(\\d+([Ee]([\\+\\-])?\\d+)?)\\b',
          name: 'constant.numeric.float.maxscript'
        },
        {
          captures: {
            1: {name: 'leading.decimal'},
            2: {name: 'decimal.separator'}
          },
          match: '\\b(\\d+)(\\.)(?!\\w)',
          name: 'constant.numeric.float.no-trailing-digits.maxscript'
        },
        {
          captures: {
            1: {name: 'decimal.separator'},
            2: {name: 'trailing.decimal'},
            3: {name: 'exponential.decimal'}
          },
          match: '(?<!\\w)(\\.)(\\d+([Ee]([\\+\\-])?\\d+)?)\\b',
          name: 'constant.numeric.float.no-leading-digits.maxscript'
        }
      ]
    },
    functions: {
      begin:
        '(?i)\\b(?:(mapped)\\s+)?(f(?:unctio)?n)\\s+(\\w+)((?:\\s*\\w+\\b(?!:))*)',
      beginCaptures: {
        1: {name: 'storage.function.modifier.maxscript'},
        2: {name: 'storage.function.type.maxscript'},
        3: {name: 'entity.name.function.maxscript'},
        4: {name: 'variable.positional.parameter.maxscript'}
      },
      end: '=',
      endCaptures: {0: {name: 'keyword.operator.maxscript'}},
      name: 'meta.function.maxscript',
      patterns: [{include: '$self'}]
    },
    int: {match: '(?<!\\$)\\b\\d+\\b', name: 'constant.numeric.int.maxscript'},
    main: {
      patterns: [
        {include: '#punctuation'},
        {include: '#parameter'},
        {include: '#strings'},
        {include: '#name'},
        {include: '#time'},
        {include: '#float'},
        {include: '#int'},
        {include: '#comments'},
        {include: '#functions'},
        {include: '#operators'},
        {include: '#constants'},
        {include: '#arrays'}
      ]
    },
    name: {
      captures: {
        1: {name: 'punctuation.definition.name.begin.maxscript'},
        2: {name: 'punctuation.definition.name.quoted.maxscript'},
        3: {name: 'punctuation.definition.name.unquoted.maxscript'}
      },
      match: "(#)(?:('[^']*')|(\\w+))",
      name: 'constant.other.name.maxscript'
    },
    operators: {
      patterns: [
        {
          match:
            "(?i)(?:(\\+=?|\\*=?|-=?|\\/=?|\\^|!?=|:|'|&|={1,2}|<=?|>=?|\\?|\\$|\\.{1,3}|\\\\))|(?:\\b(?<![\\.\\$])(and|as|by|in|not|of|or|to)(?!\\s*:)\\b)",
          name: 'keyword.operator.maxscript'
        },
        {
          match:
            '(?i)\\b(?<![\\.\\$])(animate|at|case|catch|collect|continue|do|else|exit|for|if|max|return|set|then|throw|try|undo|when|where|while|with)(?!\\s*:)\\b',
          name: 'keyword.control.maxscript'
        },
        {
          match:
            '(?i)\\b(?<![\\.\\$])(fn|function|macroscript|parameters|plugin|rcmenu|rollout|struct|tool|utility)(?!\\s*:)\\b',
          name: 'storage.type.maxscript'
        },
        {
          match:
            '(?i)\\b(?<![\\.\\$])(about|coordsys|global|local|mapped|persistent)(?!\\s*:)\\b',
          name: 'storage.modifier.maxscript'
        },
        {
          match: '(?i)\\b(?<![\\.\\$])(from)(?!\\s*:)\\b',
          name: 'keyword.reserved.maxscript'
        }
      ]
    },
    parameter: {match: '\\w+:', name: 'variable.parameter.argument.maxscript'},
    punctuation: {
      patterns: [
        {match: ';', name: 'punctuation.terminator.statement.maxscript'},
        {match: ',', name: 'punctuation.delimiter.comma.maxscript'},
        {match: '{|}', name: 'punctuation.brace.curly.maxscript'},
        {match: '\\(|\\)', name: 'punctuation.brace.round.maxscript'},
        {match: '\\[|\\]', name: 'punctuation.brace.square.maxscript'},
        {match: '\\.(?!\\d)', name: 'punctuation.delimiter.period.maxscript'}
      ]
    },
    strings: {
      patterns: [
        {
          begin: "'",
          beginCaptures: {
            0: {name: 'punctuation.definition.string.begin.maxscript'}
          },
          end: "'",
          endCaptures: {
            0: {name: 'punctuation.definition.string.end.maxscript'}
          },
          name: 'string.quoted.single.maxscript',
          patterns: [
            {
              match: "\\\\(\\*|\\?|\\\\|')",
              name: 'constant.character.escape.maxscript'
            }
          ]
        },
        {
          begin: '(?<!@)"',
          beginCaptures: {
            0: {name: 'punctuation.definition.string.begin.maxscript'}
          },
          end: '"',
          endCaptures: {
            0: {name: 'punctuation.definition.string.end.maxscript'}
          },
          name: 'string.quoted.double.maxscript',
          patterns: [
            {
              match: '\\\\("|n|r|t|\\*|\\?|\\\\|%|x[A-Fa-f0-9]+)',
              name: 'constant.character.escape.maxscript'
            }
          ]
        },
        {
          begin: '@"',
          beginCaptures: {
            0: {name: 'punctuation.definition.verbatim.string.begin.maxscript'}
          },
          end: '"',
          endCaptures: {
            0: {name: 'punctuation.definition.verbatim.string.end.maxscript'}
          },
          name: 'string.quoted.verbatim.double.maxscript'
        }
      ]
    },
    time: {
      patterns: [
        {
          match:
            '(?<!\\w|\\.)(?:\\d*\\.\\d+|\\d+\\.?\\d*)[msft](?:(?:\\d+|\\d*\\.\\d*)[msft])*',
          name: 'constant.numeric.time.maxscript'
        },
        {
          match: '(?<!\\w)\\d+:\\d*\\.\\d*(?!\\w)',
          name: 'constant.numeric.time.smpte.maxscript'
        },
        {
          match: '(?<!\\w|\\.)(?:\\d*\\.\\d+|\\d+\\.?\\d*)n(?!\\w)',
          name: 'constant.numeric.time.normalised.maxscript'
        }
      ]
    },
    variables: {
      patterns: [
        {
          match:
            '(?i)\\b(angle|bitmap|button|check(?:box|button)|colorpicker|group|(?:combo|group|multilist|list)Box|(?:curve|dotNet|Schematic)Control|dropdownList|editText|hyperLink|imgTag|label|(?:map|material|pick)button|popUpMenu|progressbar|radiobuttons|slider|spinner|SubRollout|timer)\\s+(?!:)',
          name: 'support.class.rollout-control.maxscript'
        },
        {
          match:
            '(?i)\\b(3D_Studio|3D_Studio_Shape|3D_StudioExporterPlugin|_NullMtl_Material|_NullTex|_xxxx_NullMtl_Material|A360_Cloud_Rendering|A360Renderer|ACIS_SAT|ActionPredicate|ActiveXControl|add|Add_UV_Texsurf__base|Adjustments|Adjustments__lume|Adobe_Illustrator|Adobe_Illustrator_Shape|adsk_area_shadow_samples|adsk_aspect_ratio_height_per_width|adsk_aspect_ratio_width_per_height|adsk_base_UVGenerator|adsk_bitmap|adsk_bitmaptextureMap|adsk_blurred_reflection_multiplier|adsk_blurred_refraction_multiplier|adsk_core_glossy_sampler|adsk_decal|adsk_environment|adsk_Environment_Blur|adsk_Kelvin_2_Color|adsk_KelvinToColor|adsk_Light_Default|adsk_Light_Photometric|adsk_Map_Hemispherical_Bitmap|adsk_Map_Hemispherical_Bitmap_V2|adsk_Map_Lens_Bokeh|adsk_Map_Simple_Bitmap|adsk_Map_Simple_Bitmap_V2|adsk_Map_Simple_Bumpmap|adsk_Map_Simple_Bumpmap_V2|adsk_Map_Simple_Checker|adsk_Map_Simple_Checker_V2|adsk_Map_Simple_Color_V2|adsk_Map_Simple_Colormap|adsk_Map_Simple_Colormap_V2|adsk_Map_Simple_Gradient_V2|adsk_Map_Simple_Marble|adsk_Map_Simple_Marble_V2|adsk_Map_Simple_Noise|adsk_Map_Simple_Noise_V2|adsk_Map_Simple_Speckle|adsk_Map_Simple_Speckle_V2|adsk_Map_Simple_Tile|adsk_Map_Simple_Tile_V2|adsk_Map_Simple_Wave|adsk_Map_Simple_Wave_V2|adsk_Map_Simple_Wood|adsk_Map_Simple_Wood_V2|adsk_Map_UnifiedBitmap_V2|adsk_Metal_Cutouts_Shape|adsk_Metal_Noise_Anodized|adsk_Metal_Noise_Anodized_V2|adsk_Metal_Patterns__Knurls|adsk_Metal_Patterns__Knurls_V2|adsk_Metal_switch_color_shader|adsk_Metal_switch_color_shader_V2|adsk_Mia_Optimizer|adsk_mrStoreElements|adsk_Mtl_Ceramic|adsk_Mtl_Ceramic_V2|adsk_Mtl_Concrete|adsk_Mtl_Concrete_V2|adsk_Mtl_Glazing|adsk_Mtl_Glazing_V2|adsk_Mtl_Hardwood|adsk_Mtl_Hardwood_V2|adsk_Mtl_MasonryCMU|adsk_Mtl_MasonryCMU_V2|adsk_Mtl_MatteShadowReflections|adsk_Mtl_Metal|adsk_Mtl_Metal_V2|adsk_Mtl_MetallicPaint|adsk_Mtl_MetallicPaint_V2|adsk_Mtl_Mirror|adsk_Mtl_Mirror_V2|adsk_Mtl_PlasticVinyl|adsk_Mtl_PlasticVinyl_V2|adsk_Mtl_PointCloudMaterial|adsk_Mtl_SimpleGeneric|adsk_Mtl_SimpleGeneric_V2|adsk_Mtl_SolidGlass|adsk_Mtl_SolidGlass_V2|adsk_Mtl_Stone|adsk_Mtl_Stone_V2|adsk_Mtl_WallPaint|adsk_Mtl_WallPaint_V2|adsk_Mtl_Water|adsk_Mtl_Water_V2|adsk_scale_internal_to_meters|adsk_scale_meters_to_internal|adsk_scale_meters_to_internal_inverse|adsk_Shd_Brick_spider|adsk_Shd_Ceramic|adsk_Shd_Ceramic_spider|adsk_Shd_Ceramic_V2|adsk_Shd_Concrete|adsk_Shd_Concrete_spider|adsk_Shd_Concrete_V2|adsk_Shd_Fabric_spider|adsk_Shd_Glazing|adsk_Shd_Glazing_spider|adsk_Shd_Glazing_V2|adsk_Shd_Hardwood|adsk_Shd_Hardwood_spider|adsk_Shd_Hardwood_V2|adsk_Shd_Lightmap|adsk_Shd_Lightmap_V2|adsk_Shd_MasonryCMU|adsk_Shd_MasonryCMU_spider|adsk_Shd_MasonryCMU_V2|adsk_Shd_Metal|adsk_Shd_Metal__Blender|adsk_Shd_Metal__Blender_V2|adsk_Shd_Metal_spider|adsk_Shd_Metal_V2|adsk_Shd_MetallicPaint_flakes|adsk_Shd_MetallicPaint_flakes__adsk|adsk_Shd_MetallicPaint_mia|adsk_Shd_MetallicPaint_Mia__adsk|adsk_Shd_MetallicPaint_mia_V2|adsk_Shd_MetallicPaint_Mia_V2_adsk|adsk_Shd_MetallicPaint_spider|adsk_Shd_MetallicPaint_V2|adsk_Shd_MetallicPaint_V2_adsk|adsk_Shd_MetallicPaint_V2_spider|adsk_Shd_Mirror_spider|adsk_Shd_PlasticVinyl|adsk_Shd_PlasticVinyl_spider|adsk_Shd_PlasticVinyl_V2|adsk_Shd_SolidGlass|adsk_Shd_SolidGlass_spider|adsk_Shd_SolidGlass_V2|adsk_Shd_SSS|adsk_Shd_SSS_V2|adsk_Shd_Stone|adsk_Shd_Stone_spider|adsk_Shd_Stone_V2|adsk_Shd_WallPaint|adsk_Shd_WallPaint_spider|adsk_Shd_WallPaint_V2|adsk_Shd_Water_spider|adsk_SS_Environment|adsk_Tex_Bricks|adsk_Tex_Cellular|adsk_Tex_Checker|adsk_Tex_Concrete_SmoothBumpTex|adsk_Tex_Concrete_SmoothBumpTex_V2|adsk_Tex_Gradient|adsk_Tex_Marble|adsk_Tex_Output|adsk_Tex_Smoke|adsk_Tex_Speckle|adsk_Tex_Water|adsk_Tex_Wood|adsk_ToneOpLog|adsk_Utility_Alpha_From_Shader|adsk_Utility_Bitmap|adsk_Utility_Bitmap_Tweak|adsk_Utility_bool_to_int|adsk_Utility_Bump|adsk_Utility_BumpLookup|adsk_Utility_Change_Range|adsk_Utility_Color_Correction|adsk_Utility_Color_Override|adsk_Utility_color_to_bool|adsk_Utility_color_to_booltextureMap|adsk_Utility_Color_to_Float|adsk_Utility_Color_to_Float3|adsk_Utility_ColorByObject_Switch|adsk_Utility_ColorCorrection|adsk_Utility_ColorCorrection2|adsk_Utility_ColorPassThrough|adsk_Utility_ColorShaderSwitcher|adsk_Utility_ColorState|adsk_Utility_Combine3Bump|adsk_Utility_Condition|adsk_Utility_Contrast|adsk_Utility_Curve|adsk_Utility_CurveControl|adsk_Utility_ElliptBump|adsk_Utility_ElliptTex|adsk_Utility_Falloff|adsk_Utility_Float3_to_Color|adsk_Utility_Float3_to_Float|adsk_Utility_Float_to_Color|adsk_Utility_Float_to_Float3|adsk_Utility_Gamma|adsk_Utility_GenericNoise|adsk_Utility_Int_to_Float|adsk_Utility_IntState|adsk_Utility_InvertScalar|adsk_Utility_LightmapSwitcher|adsk_Utility_LuminanceToIntensity|adsk_Utility_Mia|adsk_Utility_Mia_Basic|adsk_Utility_MiaDecomposer|adsk_Utility_Mix2Color|adsk_Utility_Mix2Shader|adsk_Utility_Mix3Bump|adsk_Utility_Mix8Shader|adsk_Utility_MultiMixer|adsk_Utility_MultiplyDivide|adsk_Utility_NodeVisibility|adsk_Utility_Noise|adsk_Utility_Noise_Tweakable|adsk_Utility_Output|adsk_Utility_PatternGenerator|adsk_Utility_ScalarShaderSwitcher|adsk_Utility_ScalarState|adsk_Utility_scale_zero_up|adsk_Utility_Shader_PassThrough|adsk_Utility_SimpleFloatMixer|adsk_Utility_SimpleMixer|adsk_Utility_State|adsk_Utility_Switch8Color|adsk_Utility_Switch8Shader|adsk_Utility_TexLookup|adsk_Utility_VectorCoords|adsk_Utility_VectorState|adsk_Utility_XYZGenerator|ADT_Category|ADT_Object_Manager|ADT_Object_Manager_Wrapper|ADT_Style|ADT_StyleComposite|ADT_SyleLeaf|ADTCategory|ADTObjMgrWrapper|AdtObjTranslator|ADTStyle|ADTStyleComp|Adv__Ray_Traced|Advanced_Lighting_Override|Advanced_Ray_traced|Affect_Region|Age_Test|Alembic_Export|Alembic_Import|AlembicCamera|AlembicContainer|AlembicExport|AlembicImport|AlembicObject|AlembicXform|alpha|AlphaMap|alphaRenderElement|AlwaysEqualFilter|Ambient_Occlusion|Ambient_Reflective_Occlusion__3dsmax|Ambient_Reflective_Occlusion__base|AmbientOcclusionBakeElement|AmountChange|Anchor|angle|AngleAxis|AngleControl|Animation_Store|AnimTrack|Anisotropic|Apollo_Effect|Apollo_Param_Container|apolloParamContainer|ArbBone|ArbBoneTrans|Arc|Arch___Design__mi|Arch__DOF___Bokeh|Arch__Environment_Blur|Arch__Exposure___Photographic|Architectural|Architectural__Environment_Portal|Architectural__Photometric_Light|Architectural__Round_Corners_Bump|Architectural__Self_Illuminator|ArchitecturalMaterial|Area|Area_Shadows|Array|ArrayParameter|AsciiExp|ASec_Element|Asset_Browser|Assign_Vertex_Colors|ATF_Alias_Import|ATF_Alias_importer|ATF_CATIA_V4_Import|ATF_CATIA_V4_importer|ATF_CATIA_V5_Import|ATF_CATIA_V5_importer|ATF_IGES_Import|ATF_IGES_importer|ATF_JT_Import|ATF_JT_importer|ATF_ProE_Import|ATF_ProE_importer|ATF_Solidworks_Import|ATF_Solidworks_importer|ATF_STEP_Import|ATF_STEP_importer|ATF_UG_NX_Import|ATF_UG_NX_importer|Atmosphere|atmosphereRenderElement|AtmosphericClass|ATSMax|Attachment|AudioClip|AudioFile|AudioFloat|AudioPoint3|AudioPosition|AudioRotation|AudioScale|Auto_Secondary_Element|AutoCADImport|AutoCam|autodesk_base|Autodesk_Map|Autodesk_Material|autodesk_point_cloud|Autodesk_Point_Cloud_Base_Shader|Autodesk_Point_Cloud_Generator|Autodesk_Point_Cloud_Material|Autodesk_Point_Cloud_Shader|AutodeskMap|AutodeskMaterial|Automatic_Exposure_Control|AutomaticAdaptiveExposureControl|AutomaticLinearExposureControl|AVI|Avoid_Behavior|AvoidBehavior|Awning|Axis_Helper|AxisHelperObj|Background|BackgroundRenderElement|bakeShell|Barycentric_Morph_Controller|Base_LayerBase_Layer|Batch_ProOptimizer|Batch_Render_Manager|Beam|Beam__lume|Beauty|beautyRenderElement|Bend|BendMod|BendModWSM|Bevel|Bevel_Profile|bezier_color|bezier_float|bezier_point3|bezier_point4|bezier_position|bezier_rgba|bezier_rotation|bezier_scale|bgndRenderElement|BiFold|BigMatrix|BigMatrixRowArray|Billboard|BinStream|Biped_Object|Biped_SubAnim|BipedCopy|BipedFSKey|BipedGeneric|BipedKey|bipedSystem|BipSlave_Control|Birth|Birth_File|Birth_Group|Birth_Paint|Birth_Script|Birth_Texture|BirthGrid|BirthGroup|BirthStream|BirthTexture|BitArray|bitmap|Bitmap_Photometric_Paths|BitmapControl|BitmapPagerData|bitmapTex|Bitmaptexture|Blackman|Blend|Blendfilter|BlendMap|BlendRenderElement|Blinn|Blinn2|Blizzard|BlobMesh|Block|Block_Control|Block_Manager_Wrapper|BlockInstanceFilter|BlockMgrWrapper|blur|Blur_Falloff|Blur_Fire___Ver__3_03|Blur_Gradient|Blur_Matte_Mtl|blur_ONB|Blur_s_Alpha_Compositor|Blur_Shellac|BlurDeflector|BlurDeflector_Modifier|BlurLib|BlurLib__Perturbation|BlurNoise|BlurWind|BlurWind_Modifier|BlurWindSpaceWarp|BMP|bmpio|Body_Cutter|Body_Join|Body_Object|Body_Utility|Bomb|Bombbinding|Bone|BoneData|BoneGeometry|BoneObj|Bones|BoneSegTrans|boolcntrl|Boolean2|boolean_float|BooleanClass|BoolPacket|Box|Box2|Box3|Box_2_Global_Utility|BoxGizmo|Bricks|Bricks__3dsmax|Brightness_and_Contrast|briteCon|builtin_displace_mdl|builtin_function_mdl|builtin_mdl_multiply_scalar|Bulge_Angle_Deformer|Bump__3dsmax|Bump_Basis__base|Bump_Capture__lume|Bump_Combiner__adsktextureMap|Bump_Map__base|BumpCapture|ButtonControl|C_Ext|Cache|Cache_Disk|Cache_Selective|CacheDisk|CacheFile|CacheSelective|Caddy|Camera_Culling|Camera_IMBlur|Camera_Map_Per_Pixel|Camera_Match|Camera_Tracker|CameraCulling|CameraMap|CameraMapSpaceWarp|CameraMapTexture|CameraMBlur|CamMatchDataCustAttrib|Camoflage|CamPoint|Cap_Holes|Capsule|Captured_Object_State|Car_Paint_Material__mi|Car_Paint_Shader__mi|Casement|Cast_Shadows_Only|CAT_LiftOffset|CAT_LiftPlantMod|CATBone|CATBoneData|CATBoneDataMatrix3Controller|CATBoneSegTrans|CATClipFloat|CATClipMatrix3|CATClipRoot|CATClipWeights|CATCollarBone|CATDigitSegTrans|CATDummyMoveMask|CATFootBend|CATFootLift|CATFootTrans2|CATGizmoTransform|CATHDPivotTrans|CATHierarchyBranch|CATHierarchyBranch2|CATHierarchyLeaf|CATHierarchyRoot|CATHIPivotTrans|CATKneeAngle|CATLegWeight|CATLiftOffset|CATLiftPlantMod|CATLimbData2|CATLimbData2FloatController|CATMonoGraph|CATMotionDigitRot|CATMotionHub2|CATMotionLayer|CATMotionLimb|CATMotionPlatform|CATMotionRot|CATMotionRotRotationController|CATMotionTail|CATMotionTailRot|Catmull_Rom|CATMuscle|CATp3|CATParent|CATParentTrans|CATPivotPos|CATPivotRot|CATPoint3|CATRigRootNodeCtrl|CATSpineData2|CATSpineData2FloatController|CATSpineTrans2|CATStepShape|CATTransformOffset|CATUnitsPosition|CATUnitsScale|CATWeight|CATWeightShift|ccCurve|ccPoint|CCRootClass|Cellular|Cellular__3dsmax|cellularTex|Chair|Chamfer|ChamferBox|ChamferCyl|ChamferMod|ChangeHandler|channel|Channel_Info|Character|CharacterHelper|CharStream|CheckBoxControl|CheckButtonControl|Checker|Checker__3dsmax|CIN|Circle|Civil_View_Divide_Spline|Civil_View_Guard_Rail|Civil_View_Path___Surface_Constraint|Civil_View_Path___Surface_ConstraintMatrix3Controller|Civil_View_Path___Surface_ConstraintMatrix3ControllerMatrix3Controller|Civil_View_Road_Marking|Civil_View_Sight_Checker__Calc|Civil_View_Spline_to_Mesh|Civil_View_Swept_Object|Civil_View_Traffic_Data_Constraint|class|Clean_MultiMaterial|CleanUp|Clip_Associations|ClipAssigner|ClipAssociation|ClipState|Clone_and_Align_Tool|Cloth|clothfx|CMB|CogControl|collapse|CollarBoneTrans|Collision|Collision_Spawn|color|Color_Alpha__base|Color_Average__base|Color_Balance|Color_Clamp|Color_Clipboard|Color_Correction|Color_Correction__adsk|Color_Intensity__base|Color_Interpolate__base|Color_Mix__base|Color_Override_Ray_Type_Switcher|Color_RGB|Color_RGBA|Color_Spread__base|colorBalance|ColorCorrection|ColorCorrection__3dsmax|ColorPickerControl|colorReferenceTarget|COM_DCOM_Server_Control|Combi__contour|ComboBoxControl|Combustion|Compass|Complete_Composite|CompleteMap|Composite__3dsmax|CompositeMap|compositematerial|compositeTexture|CompositeTexturemap|Condition|Cone|Cone_Angle|ConeAngleManip|Conform|ConformSpaceWarp|Connect|Container|ContainerHelper|contour_composite|Contour_Composite__contour|contour_contrast_function_levels|Contour_Contrast_Function_Levels__contour|contour_contrast_function_simple|Contour_Contrast_Function_Simple__contour|contour_contrast_light_levels|Contour_Data_Packet|contour_only|Contour_Only__contour|contour_ps|Contour_PS__contour|contour_shader_combi|contour_shader_curvature|contour_shader_depthfade|contour_shader_factorcolor|contour_shader_framefade|contour_shader_layerthinner|contour_shader_lightlines|contour_shader_maxcolor|contour_shader_silhouette|contour_shader_simple|contour_shader_widthfromcolor|contour_shader_widthfromlight|contour_shader_widthfromlightdir|contour_store_function|Contour_Store_Function__contour|contour_store_function_simple|Contour_Store_Function_Simple__contour|contour_store_light|Contour_Translator|contrast|Control|ControlContainer|ControlContainerGeometry|Convert|ConvertToPatch|Cook_Variable|Cookie|Copy_Out|CopyCollection|Crease|CreaseMod|CreaseSet|CreaseSetMod|Create_Out_of_Range_Keys|CrosFade|CrossSection|Crowd|CrowdAssignment|CrowdDelegate|CrowdGroup|CrowdState|CrowdTeam|CrowdTransition|CTBitMap|CTMotionTracker|cubic|Cubic_Morph_Controller|Curvature__contour|CurveClass|CurveControl|CurveCtlGeneric|CurvePointsArray|CustAttribContainer|Custom_LPE|CV_Curve|CV_Curveshape|CV_Surf|CylGizmo|Cylinder|D6Joint|DAEEXP|DaeExporter|DAEIMP|DaeImporter|Damper|Data_Icon|Data_Operator|Data_Test|DataOpDeleteCatcher|DataOperator|DataOperIcon|DataPair|DataTest|DataTestIcon|DataViewGroup|Daylight|Daylight_Slave_Controller|Daylight_Slave_Intensity_Controller|DaylightAssemblyHead|DDS|DecayNoise|DecayNoiseSpaceWarp|Default_Color_Picker|Default_Scanline_Renderer|Default_Sound|DefaultScanlineRenderer|DefaultSource|Deflector|Deflectorbinding|Deform_Curve|Deformable_gPoly|Delegate|DeleteMesh|DeleteParticles|DeletePatch|DeleteSplineModifier|Dent|Dent__3dsmax|dents|Depth_Fade__contour|Depth_of_Field|Depth_of_Field___Bokeh|Depth_of_Field__mental_ray|Depth_of_FieldMPassCamEffect|dgs_material|DGS_Material__3dsmax|DGS_Material__physics|DGS_Material__physics_phen|dgs_material_phen|dgs_material_photon|DGS_Material_Photon__3dsmax|DGS_Material_Photon__physics|DialogMonitor|Dielectric__base|dielectric_material|Dielectric_Material__3dsmax|Dielectric_Material__physics|dielectric_material_phen|dielectric_material_photon|Dielectric_Material_Photon__3dsmax|Dielectric_Material_Photon__physics|Diffuse|diffuseMap|diffuseRenderElement|DigitData|DigitDataFloatController|DigitSegTrans|Directionallight|DirectX_9_Shader|DirectX_9_ShaderReferenceTarget|DirectX_Shader_Default|Dirt|Discreet_Radiosity|DiscreetRadiosityMaterial|Discretizator|Disp_Approx|Displace|Displace_Mesh|Displace_NURBS|Displacebinding|Displacement_3D__3dsmax|Display_Script|DisplayCulling|DisplayData|DisplayParticles|DisplayScriptParticles|Distance_Blender|Distortion|Distortion__lume|Do_Nothing_Texture|DOFEffect|Donut|dotNetClass|dotNetControl|dotNetMethod|dotNetMXSValue|dotNetObject|Double|DoublePacket|DoubleSided|doubleSidedMat|Drag|DragMod|Drip|DripTexmap|DSIMDSCC|Dummy|DummyRadMapClass|DummyUnknown_0X00000B20|DustDevil_V2_07|DWF_Exporter|DWG_Export|DWG_ExportExporterPlugin|DwgAlwaysEqualFilter|DwgBlockDefinitionFilter|DwgBlockInsAsNodeHierarchyFilter|DwgBlockInsertFilter|DwgCameraPacket|DwgColorFilter|DwgColorMaterialPacket|DwgColorSplitByMaterialFilter|DwgEnhancedLayerFilter|DwgEnhColorPacket|DwgEnhLayerPacket|DwgEntityPacket|DwgExtrusionFilter|DwgExtrusionPacket|DwgFactory|DwgFilterList|DwgGridPacket|DwgHandleFilter|DwgLayerFilter|DwgLayerTable|DwgLightPacket|DwgMaterialFilter|DwgMaterialFilterReferenceMaker|DwgMaterialPacket|DwgMaterialPacketReferenceMaker|DwgPluginTranslatorForwardingFilter|DwgPointTrans|DwgSunPacket|DwgTableEntryPacket|DwgTableRecord|DxMaterial|DynDiv|DynGRail|DynMarks|DynoSkin|DynSMesh|DynSOS|DynXFCC|DYNXFCCM3|DynXFCCV3|Ease|edge|Edge__lume|Edge_Shadow__lume|EdgeSelection|EdgeShadow|EdgesTex|Edit_Mesh|Edit_Normals|Edit_Patch|Edit_Poly|Edit_Spline|Editable_mesh|Editable_Patch|Editable_Poly|EditablePolyMesh|EditNormals|EditPolyMod|EditTextControl|EffectClass|Egg|Electric|Ellipse|emissionRenderElement|Empty_Flow|EmptyClass|EmptySource|Emulator|Environment__3dsmax|Environment_Background_Camera_Map__mi|Environment_Background_Switcher__mi|Environment_Blur|Environment_Probe_Chrome_Ball__mi|Environment_Probe_Gray_Ball__mi|EPS_Image|Euler_Filter|Euler_XYZ|EulerAngles|Event|ExportHTR|Expose_Euler_Control|Expose_Float_Control|Expose_Point3_Control|ExposeTm|ExposeTransform|exposurectrl|Express_Save|ExpressSave|Extrude|Facade|Facade__lume|Face_Extrude|Faces_Orientation|FaceSelection|Factor_Color__contour|Fade|falloff|Falloff2__3dsmax|Falloff__3dsmax|Falloff_Manipulator|FalloffManip|fallofftextureMap|FBXEXP|FbxExporter|FBXIMP|FbxImporter|FbxMaxByMaterialFilter|FbxMaxByObjectFilter|FbxMaxByRevitCategoryFilter|FbxMaxByRevitTypeFilter|FbxMaxFactory|FbxMaxFilterList|FbxMaxObjTranslator|FbxMaxOneObjectFilter|FbxMaxRevitFactory|FbxMaxTableRecord|FbxMaxTableRecordReferenceMaker|FbxMaxWrapper|FbxRevitMaxTableRec|FFD2x2x2|FFD3x3x3|FFD4x4x4|FFD_2x2x2|FFD_3x3x3|FFD_4x4x4|FFD_Binding|FFD_Select|FFDBox|FFDCyl|File_Link_Manager|File_Output|FileLink_DwgLayerTable|FileLink_LinkTable|FileLink_VzMaterialTable|FileLinkAsDwgImporter|fileOut|FileStream|Fillet_Chamfer|Film_Grain|FilmGrain|Filter_kernel_plug_in_not_found|Find_Target|Fire_Effect|Fire_Effect__3dsmax|Fix_Ambient|FixAmbient|Fixed|FlakesMtl|Flat_Mirror|Flat_Mirror__3dsmax|flatMirror|Flex|Flight_Studio|Flight_Studio_Bitmap_Class_Name|flightstudioimage|Flipped_UVW_Faces|FlippedFacesClass|FlippedUVWFacesClass|float|Float_Expression|Float_Layer|float_limit|float_list|float_ListDummyEntry|Float_Mixer_Controller|Float_Motion_Capture|Float_Reactor|float_script|Float_Wire|Float_XRef_Controller|FloatLimitCtrl|FloatList|FloatPacket|FloatReactor|FloatXRefCtrl|Flow|FltGUP|FltImport|FltTextureAttrCustAttrib|Fog|Fog__3dsmax|FogHelper|Foliage|FoliagetextureMap|Follow_Bank|FootBend|FootLift|Footsteps|FootTrans|fopenexr|Force|Frame_Fade__contour|Free_Area|Free_Cylinder|Free_Disc|Free_Light|Free_Linear|Free_Point|Free_Rectangle|Free_Sphere|Freecamera|freeIesSun|freeSpot|Function|FunctionReferenceTarget|GameNavGup|Garment_Maker|Garmentize2|Gas_Giant|GBuffer_to_RGB|Generic|Gengon|Geo_Cone__base|Geo_Cube__base|Geo_Cylinder__base|Geo_Instance__base|Geo_Instnace_mlist__base|Geo_Plane__base|Geo_Sphere__base|Geo_Torus__base|GeometryCheckerManager|geometryReferenceTarget|GeomObject|GeoSphere|GIF|Gimp_Oilify|gizmoBulge|gizmoJoint|gizmoJointMorph|Glare|Glare__lume|GlaretextureMap|Glass|Glass__lume|Glass__physics_phen|Global_Clip_Associations|Global_Container|Global_Motion_Clip|GlobalClipAssociation|GlobalMotionClip|Glow|Glow__lume|Glow_Element|Gnormal|Go_To_Rotation|Gradient|Gradient__3dsmax|Gradient_GradCtlData|Gradient_Ramp|Gradient_Ramp__3dsmax|GRAIN|gravity|Gravitybinding|grid|GripManager|Group_Operator|Group_Select|GroupBoxControl|GroupEndControl|GroupOperator|GroupSelection|GroupStartControl|Hair_Atmospheric|Hair_Atmospheric_Gizmo|Hair_GI_Atmospheric|Hair_Instanced_Geometry_MR_Shader|Hair_MR_Geom_Shader|Hair_MR_Object|HairAtmospheric|HairAtmosphericGizmo|HairEffect|HairGIAtmospheric|HairLightAttr|HairMaxUtility|HairMod|HairMRGeomShader|HairMRIntanceGeomShader|HairMRObj|HairRenderElement|HairVrObject|HalfRound|HashTable|HdlObjObj|HdlTrans|HDR_Image_Motion_Blur__mi|HDRI|Hedra|Height_Map_Displacement__3dsmax|HeightMap|Helix|HiddenUnselectedNodeCache|HighlightOnly|HKey|Hose|Hotspot_Manip|HotspotManip|HSDS|HSDS_Modifier|HSDSObject|HSUtil|Hub|HubObject|HubTrans|icon|IES_Sky|IES_Sun|IesSkyLight|IFL|IFL_Manager|IGES_Export|IK_Chain_Object|IK_Controller|IK_ControllerMatrix3Controller|IK_Position_Controller|IK_Spline_End_Twist_Manip|IK_Spline_Start_Twist_Manip|IK_Swivel_Manip|IKChainControl|IKControl|IKEndSpTwistManip|IKHISolver|IKLimb|IKStartSpTwistManip|IKSwivelManip|IKTarget|IKTargetObj|IKTargTrans|Illum_Blinn__base|Illum_CookTorr__base|Illum_Hair__base|Illum_Lambert__base|Illum_Phong__base|Illum_Ward__base|Illum_Ward_Deriv__base|Illuminance_HDR_Data|Illuminance_Render_Element|IlluminanceRenderElement|Illumination|Illumination__lume|Illumination_Render_Element|illumRenderElement|imageMotionBlur|ImgTag|ImportHTR|ImportTRC|IndirectRefTargContainer|Initial_State|InitialState|Ink|Ink__N_Paint__3dsmax|InkNPaint|InkRenderElement|Inline|Input_mParticles|Input_Proxy|InputCustom|InputPhysX|InputProxy|InputStandard|Instance_Duplicate_Maps|Instance_Manager_Wrapper|InstanceDuplMap|InstanceMgrWrapper|Int64Packet|integer|Integer64|IntegerPtr|Interface|InterfaceFunction|Interval|IntPacket|InventorImport|Inverse_of_Photographic_Exposure|Invisible__physics_phen|IObject|iray__Alpha|iray__Irradiance|iray__Normal|iray_Matte_Environment|iray_Plugin_Options|iray_Renderer|iray_Section|irayAlpha|irayAreaLights|irayCaustics|irayCustom|irayDiffuse|irayEnvLight|irayIrradiance|irayNormal|irayPointLights|irayReflection|iraySelfIllum|irayTranslucency|irayTransparency|Isolated_Vertices|IsolatedVertexClass|JAngleData|JBinaryData|JBoolData|JColor3Data|JColorData|JFlagCtlData|JFlagSetData|JFloat3Data|JFloatData|JGradCtlData|Join_Bodies|Joint_Angle_Deformer|Joystick_Input_Device|JPEG|jpegio|JPercent3Data|JPercentData|JSubtex|JWorld3Data|JWorldData|Keep_Apart|Kelvin_Temperature_Color|KneeAngle|L_Ext|L_Type_Stair|LabelControl|Landscape|Landscape__lume|LandscapetextureMap|LandXML___DEM_Model_Import|LandXMLImport|Lathe|Lattice|Layer_Manager|Layer_Output|Layer_Thinner__contour|LayerFloat|LayerInfo|LayerMatrix3|LayerRoot|LayerTransform|LayerWeights|LegWeight|Lens_Clamp__base|Lens_Effects|Lens_Effects_Flare_Filter|Lens_Effects_Focus_Filter|Lens_Effects_Glow_Filter|Lens_Effects_Hilight_Filter|Lens_Stencil__base|Level_of_Detail|Light__Area|Light__Environment|Light__Point|Light_Boy|Light_Infinite__base|Light_lines|Light_Photometric__base|Light_Point__base|Light_Spot__base|Light_Tracer|LightChannelClass|Lighting|Lighting_Analysis_Data|Lighting_Analysis_Overlay|lightingAnalysisDataRenderElement|LightingAnalysisOverlay|LightingMap|LightingRenderElement|LightMap|Lightmap_Sample__base|Lightmap_Write__base|LightMeter|LightscapeExposureControl|LightTrace|LimbData2|line|linear|Linear_Exposure_Control|linear_float|linear_position|linear_rotation|linear_scale|LinearShape|Lines|Link|Link_Constraint|Link_Inheritance__Selected|Link_Transform|LinkBlockInstance|LinkBlockInstanceshape|LinkComposite|LinkCompositeshape|LinkControl|Linked_XForm|LinkedXForm|LinkLeaf|LinkLeafshape|LinkOriginPtHelper|LinkTimeControl|ListBoxControl|Local_Euler_XYZ|Lock_Bond|LockedMapWrapper|LockedMaterialWrapper|LockedModifierWrapper|LockedObjectWrapper|LockedObjectWrapper_Obsolete|LOD|LOD_Controller|Loft|LoftObject|Logarithmic_Camera_Exposure__3dsmax|Logarithmic_Exposure_Control|lookat|LookAt_Constraint|Lookup_Background__base|Lookup_Cube1__base|Lookup_Cube6__base|Lookup_Cylindrical__base|Lookup_Spherical__base|LTypeStair|Lume_Glow_Material__Lambertian|Lume_Metal_Material__Phong|Lume_Translucent_Material__Lambertian|Luminaire|LuminaireHelper|Luminance_HDR_Data|Luminance_Render_Element|LuminanceRenderElement|Lumination_Render_Element|lumRenderElement|LZFlare_AutoSec_Base|LZFlare_AutoSec_Data|LZFlare_Aux_Data|LZFlare_Data|LZFlare_Glow_Data|LZFlare_Inferno_Data|LZFlare_ManSec_Base|LZFlare_ManSec_Data|LZFlare_Prefs_Data|LZFlare_Rays_Data|LZFlare_Rend_Data|LZFlare_Ring_Data|LZFlare_Star_Data|LZFlare_Streak_Data|LZFocus_Data|LZGlow_Aux_Data|LZGlow_Data|LZGlow_Rend_Data|LZHilight_Aux_Data|LZHilight_Data|LZHilight_Rend_Data|MACUtilities|Maelstrom|MaelstromMap|MaelstromOSM|MaelstromSpaceWarp|MaelstromSpacewarpModifier|MaelstromWSM|MaelstromWSMObject|Manual_Secondary_Element|Map_to_Material_Conversion|MapButtonControl|MapChannelAdd|MapChannelDelete|MapChannelPaste|MappedGeneric|MappedPrimitive|mapping|Mapping_Object|MappingObject|MapScaler|MapScalerModifier|MapScalerOSM|MapScalerSpaceWarp|MapScalerSpacewarpModifier|MapSupportClass|Marble|Marble__3dsmax|Mask|Mask__3dsmax|maskTex|MassFX_Loader_Linkage|MassFX_RBody|Master_Controller_plugin_not_found|Master_Motion_Clip|Master_Point_Controller|MasterBlock|MasterClip|MasterList|Material_Dynamic|Material_Editor|Material_EditorReferenceMaker|Material_Frequency|Material_ID|Material_Static|Material_to_Shader|MaterialByElement|MaterialCategory|materialhelper|materialIDRenderElement|MaterialLibrary|Materialmodifier|MaterialPreviewer|MaterialPreviewerGUP|Matrix3|Matrix3_Mixer_Controller|Matte|Matte_Shadow_Reflection__mi|MatteRenderElement|MatteShadow|max2_dgs_material|max2_dgs_material_photon|max2_dielectric_material|max2_dielectric_material_photon|max2_parti_volume|max2_parti_volume_photon|max_AlphaBakeElement|max_amb_occlusion|max_ArchitecturalMaterial|max_base_AnisotropicIllumination|max_base_BlinnIllumination|max_base_ConstantIllumination|max_base_Curve|max_base_CurveControl|max_base_GBuffer_lens|max_base_ImageCollector|max_base_MeditBackground|max_base_MetalIllumination|max_base_MultiLayerIllumination|max_base_OrenNayarBlinnIllumination|max_base_Output|max_base_PhongIllumination|max_base_StraussIllumination|max_base_TranslucentIllumination|max_base_UVGenerator|max_base_VideoColorCorrect|max_base_VideoSuperBlack|max_base_WardIllumination|max_base_XYZGenerator|max_Bitmap|Max_Bitmap__3dsmax|max_BlendBakeElement|max_BlendMaterial|max_BlendMaterial_Contour|max_Bricks|max_Bump|max_CameraMap|max_Cellular|max_Checker|max_ColorCorrection|max_CompleteBakeElement|max_Composite|max_CompositeMaterial|max_CompositeMaterial_Contour|max_default_mtl_phen|max_DefaultLight|max_DefaultMaterial|max_DefaultShadow|max_Dent|max_DiffuseBakeElement|max_DirectionalLight|max_Displacement|max_DoubleSidedMaterial|max_DoubleSidedMaterial_Contour|max_dummy|max_Environment|max_Falloff|max_Falloff2|MAX_File_Finder|max_FireEffect|max_FlatMirror|max_Fog|max_GenericBakeElement|max_GizmoData|max_Glare|max_GNormal|max_Gradient|max_GradientRamp|max_HeightBakeElement|max_HeightMapDisplacement|max_IesSkylight|max_InkNPaint|max_is_material_editor|max_Landscape|max_LightBakeElement|Max_LightMap_Lightmeter_Shader|max_LightMeterLightMap|max_LightMeterMaterial|max_Marble|max_Mask|Max_Master_Clip|max_MaterialToShader|max_MatteMaterial|max_mia_material_renderelements|max_Mix|Max_MotionClip_Implementation|max_mrRenderElement|max_mrStoreElement|max_mrStoreElements|max_MultiMaterial|max_MultiMaterial_Contour|max_MultiSubMap|max_NodeData|max_NodeVisibility|max_Noise|max_NormalsBakeElement|max_ObjectData|max_OmniLight|max_Output|max_ParticleAge|max_ParticleBlur|max_ParticleData|max_PerlinMarble|max_PhotometricPointLight|max_physical_camera_lens|max_PhysicalSun|max_Planet|max_Raytrace|max_RaytraceMaterial|max_ReflectRefract|max_ReflectRefractBakeElement|max_RenderElementHandler|max_RGBMultiplier|max_rtt_lens|max_rtt_output|max_RTTCageData|max_SceneData|max_ShaderList|max_ShadowBakeElement|max_ShadowWrapper|max_ShellacMaterial|max_ShellacMaterial_Contour|max_Smoke|max_Speckle|max_SpecularBakeElement|max_Splat|max_SpotLight|max_StandardMaterial|max_StdRenderElements|max_Stucco|max_Swirl|max_TexturedSkylight|max_ThinWallRefraction|max_Tint|max_ToneOpLog|max_ToneOpPhoto|max_ToneOpPseudo|max_TopBottomMaterial|max_TopBottomMaterial_Contour|max_UnknownMaterial|max_UVCoordinate|max_vdm|max_Vectormap|max_VertexColor|max_VolumeFog|max_VolumeLight|max_Water|max_WireColor|max_Wood|max_XYZCoordinate|MAXAKey|MAXClass|MAXCurveCtl|MAXCustAttrib|MAXCustAttribArray|MAXFilterClass|MAXKey|MAXKeyArray|MAXMeshClass|MAXModifierArray|MaxMotionClipImp|MAXNoteKey|MAXNoteKeyArray|MAXObject|MAXRefTarg|MaxRenderer_COM_Server|MAXRootNode|MAXRootScene|MAXScript|MAXScriptFunction|MaxscriptParticleContainer|MAXSuperClass|MAXTVNode|MAXTVUtility|Mb_select|mCloth|Measure|Melt|mental_ray|mental_ray__Area_Light_custom_attribute|mental_ray__Displacement_Custom_Attributes|mental_ray__Indirect_Illumination_custom_attribute|mental_ray__light_shader_custom_attribute|mental_ray__material_custom_attribute|mental_ray__material_custom_attributes_manager|mental_ray__object_custom_attributes_manager|mental_ray_Import|mental_ray_iray_Renderer|mental_ray_material_override_options|mental_ray_Proxy|mental_ray_renderer|mental_ray_Shadow_Map|mentalraySun|menuitem|Mesh_Select|MeshCollision|MeshDeformPW|Mesher|meshGrid|MeshProjIntersect|MeshSelect|meshsmooth|Metal|Metal2|Metal__lume|Metal_Bump|Metal_Bump9|MetalShader|MetaSLProxyMaterial|Metronome|mi_bump_flakes|mi_car_paint_mtl_phen_max|mi_car_paint_phen|mi_car_paint_phen_max|mi_car_paint_phen_x|mi_Glossy_Reflection__mi|mi_Glossy_Refraction__mi|mi_metallic_paint|mi_metallic_paint_output_mixer|mi_metallic_paint_x|mia_ciesky|mia_cieskylight|mia_envblur|mia_exposure_photographic|mia_exposure_photographic_rev|mia_exposure_simple|mia_lens_bokeh|mia_light_surface)\\b',
          name: 'support.class.core.maxscript'
        },
        {
          match:
            '(?i)\\b(mia_material|mia_material_x|mia_photometric_light|mia_physicalsky|mia_physicalskylight|mia_physicalsun|mia_portal_light|mia_roundcorners|mib_amb_occlusion|mib_bent_normal_env|mib_blackbody|mib_bump_basis|mib_bump_map|mib_bump_map2|mib_cie_d|mib_color_alpha|mib_color_average|mib_color_intensity|mib_color_interpolate|mib_color_mix|mib_color_spread|mib_continue|mib_dielectric|mib_fast_occlusion|mib_fg_occlusion|mib_geo_add_uv_texsurf|mib_geo_cone|mib_geo_cube|mib_geo_cylinder|mib_geo_instance|mib_geo_instance_mlist|mib_geo_sphere|mib_geo_square|mib_geo_torus|mib_glossy_reflection|mib_glossy_refraction|mib_illum_blinn|mib_illum_cooktorr|mib_illum_hair|mib_illum_lambert|mib_illum_phong|mib_illum_ward|mib_illum_ward_deriv|mib_lens_clamp|mib_lens_stencil|mib_light_infinite|mib_light_photometric|mib_light_point|mib_light_spot|mib_lightmap_sample|mib_lightmap_write|mib_lookup_background|mib_lookup_cube1|mib_lookup_cube6|mib_lookup_cylindrical|mib_lookup_spherical|mib_map_get_color|mib_map_get_integer|mib_map_get_integer_array|mib_map_get_scalar|mib_map_get_scalar_array|mib_map_get_transform|mib_map_get_vector|mib_opacity|mib_passthrough_bump_map|mib_photon_basic|mib_ray_marcher|mib_reflect|mib_refract|mib_refraction_index|mib_shadow_transparency|mib_texture_checkerboard|mib_texture_filter_lookup|mib_texture_filter_lookup_2|mib_texture_filter_lookup_3|mib_texture_lookup|mib_texture_lookup2|mib_texture_polkadot|mib_texture_polkasphere|mib_texture_remap|mib_texture_rotate|mib_texture_turbulence|mib_texture_vector|mib_texture_wave|mib_transparency|mib_twosided|mib_volume|MIDI_Device|MinMaxAvg|mip_binaryproxy|mip_cameramap|mip_card_opacity|mip_fgshooter|mip_fgshooter__mi|mip_gamma_gain|mip_grayball|mip_matteshadow|mip_matteshadow_mtl|mip_mirrorball|mip_motion_vector|mip_motionblur|mip_rayswitch|mip_rayswitch_advanced|mip_rayswitch_environment|mip_rayswitch_stage|mip_rayswitch_stage__mi|mip_render_subset|mirror|Missing_Atmospheric|Missing_Camera|Missing_Custom_Attribute_Plugin|Missing_Exposure_Control|Missing_Float_Control|Missing_GeomObject|Missing_Helper|Missing_Light|Missing_Matrix3_Control|Missing_Morph_Control|Missing_Mtl|Missing_OSM|Missing_Point3_Control|Missing_Point4_Control|Missing_Position_Control|Missing_Radiosity|Missing_RefMaker|Missing_RefTarget|Missing_Render_Effect|Missing_Render_Element_Plug_in|Missing_Renderer|Missing_Rotation_Control|Missing_Scale_Control|Missing_Shader_Plug_in|Missing_Shadow_Type|Missing_Shape|Missing_SoundObj|Missing_System|Missing_Texture_Bake_Element|Missing_Texture_Output_Plug_in|Missing_TextureMap|Missing_UVGen|Missing_UVW_Coordinates|Missing_WSM|Missing_WSM_Object|Missing_XYZGen|MissingUVCoordinatesClass|misss_call_shader|misss_fast_shader|misss_fast_shader2|misss_fast_shader2_x|misss_fast_shader_x|misss_fast_simple_phen|misss_fast_skin_phen|misss_fast_skin_phen_d|misss_lambert_gamma|misss_lightmap_phen|misss_lightmap_write|misss_mia_skin2_phen|misss_mia_skin2_phen_d|misss_mia_skin2_surface_phen|misss_physical|misss_physical_phen|misss_set_normal|misss_skin_specular|Mist|Mist__lume|Mitchell_Netravali|Mix|Mix__3dsmax|mixamo|mixamo_ctrl|MixamoController|Mixer|MixinInterface|mixTexture|MMClean|MocapLayer|MocapLayerInfo|ModifierClass|MoFlow|MoFlowScript|MoFlowSnippet|MoFlowTranInfo|MoFlowTransition|MonoGraph|Morph|Morph_Angle_Deformer|MorphByBone|Morpher|MorpherMaterial|Motion_Blur|Motion_BlurMPassCamEffect|Motion_Capture|Motion_Clip|Motion_Clip_SlaveFloat|Motion_Clip_SlavePoint3|Motion_Clip_SlavePos|Motion_Clip_SlaveRotation|Motion_Clip_SlaveScale|Motion_ClipFloatController|Motion_Vector_Export__mi|MotionRenderElement|MotionTracker|Motor|MotorMod|Mouse_Input_Device|MouseTool|mP_Buoyancy|mP_Collision|mP_Drag|mP_Force|mP_Glue|mP_InterCollision|mP_Shape|mP_Solvent|mP_Switch|mP_World|mP_Worldhelper|mParticles_Flow|MPassCamEffectClass|MPG|mr_A_D_Level__Diffuse|mr_A_D_Level__Opacity|mr_A_D_Level__Reflections|mr_A_D_Level__Specular|mr_A_D_Level__Translucency|mr_A_D_Level__Transparency|mr_A_D_Output__Beauty|mr_A_D_Output__Diffuse_Direct_Illumination|mr_A_D_Output__Diffuse_Indirect_Illumination|mr_A_D_Output__Opacity_Background|mr_A_D_Output__Reflections|mr_A_D_Output__Self_Illumination|mr_A_D_Output__Specular|mr_A_D_Output__Translucency|mr_A_D_Output__Transparency|mr_A_D_Raw__Ambient_Occlusion|mr_A_D_Raw__Diffuse_Direct_Illumination|mr_A_D_Raw__Diffuse_Indirect_Illumination|mr_A_D_Raw__Opacity_Background|mr_A_D_Raw__Reflections|mr_A_D_Raw__Specular|mr_A_D_Raw__Translucency|mr_A_D_Raw__Transparency|mr_A_D_Xtra__Diffuse_Indirect_Illumination_with_AO|mr_Ambient_Occlusion|mr_Binary_Proxy|mr_Card_Opacity|mr_Labeled_Element|mr_Labeled_ElementRenderElement|mr_Matte_Shadow_Reflection|mr_Matte_Shadow_Reflection_Mtl|mr_Photographic_Exposure_Control|mr_Physical_Sky|mr_Proxy|mr_Raytype_Switcher|mr_Raytype_Switcher__advanced|mr_Shader_Element|mr_Sky|mr_Sky_Portal|mr_Sun|mrADBeauty|mrADDiffuseDirectIllumRaw|mrADDiffuseDirectIllumResult|mrADDiffuseIndirectIllumRaw|mrADDIffuseIndirectIllumResult|mrADDiffuseIndirectIllumWithAO|mrADDiffuseLevel|mrADIdirectAO|mrADOpacity|mrADOpacityBackgroundRaw|mrADOpacityBackgroundResult|mrADReflectLevel|mrADReflectRaw|mrADReflectResult|mrADRefractLevel|mrADRefractRaw|mrADRefractResult|mrADSelfIllum|mrADSpecularLevel|mrADSpecularRaw|mrADSpecularResult|mrADTranslucentLevel|mrADTranslucentRaw|mrADTranslucentResult|mrAmbientOcclusionRenderElement|mrArbRenderElement|mrArchMaterial|mrArchMaterial_Shader|mrArchMaterial_Shader_X|mrArchMaterialtextureMap|mrAreaLightCustAttrib|mrDOF|mrIICustAttrib|mrImporter|mrLightShaderCustAttrib|mrMaterial|mrMaterialCustAttrib|mrMaterialCustAttrib_Manager|MrmMod|mrNamedRenderElement|mrObjectCustAttrib_Manager|mrOptions|mrPBParameter|mrPhysSkyLight|mrShadowMap|mrSkyEnv|mrSkylightPortal|mrToneMapExposureControl|MSBipedRootClass|MSComMethod|MSCustAttribDef|MSDispatch|MSec_Element|MSEmulator|MSPluginClass|Mtl__Caustics|Mtl__Diffuse|Mtl__Reflections|Mtl__Self_Illumination|Mtl__Translucency|Mtl__Transparency|MtlBaseLib|MtlButtonControl|MtlLib|MultDeleg|Multi_Blender|Multi_Layer|Multi_Sub_Map|MultiChannelMatte|MultiLayer|MultiListBoxControl|Multimaterial|Multimaterial_empty|MultiMaterialClass|MultiMatteElement|MultiOutputChannelTexmapToTexmap|multiPassDOF|multiPassMotionBlur|Multiple_Delegate_Settings|Multiple_Delegates|Multiple_Edges|MultipleEdgeClass|MultipleFSParams|MultiRes|multiSubMaterial|multiSubMaterial_empty|Muscle_Handle|Muscle_Strand|MusclePatch|MuscleStrand|MXClip|MXSParticleContainer|MXTrack|MXTrackgroup|MyBricks|MyBricksPhenomenon|name|NamedSelSetList|NavInfo|NCurve_Sel|negative|Ngon|Night|Night__lume|NLAInfo|Node_Visibility|NodeChildrenArray|NodeCustAttrib|NodeEventCallback|NodeGeneric|NodeMonitor|NodeNamedSelSet|NodeObject|NodeTransformMonitor|Noise|Noise__3dsmax|Noise_float|Noise_point3|Noise_position|Noise_rotation|Noise_scale|Noisemodifier|NoMaterial|Normal_Bump|Normalize_Spl|Normalize_Spline|Normalmodifier|NormalsMap|Notes|NotesReferenceTarget|NoteTrack|NoTexture|NoValueClass|NSurf_Sel|NullFilter|Number|NURBS1RailSweepSurface|NURBS2RailSweepSurface|NURBS_Imported_Objects|NURBSBlendCurve|NURBSBlendSurface|NURBSCapSurface|NURBSChamferCurve|NURBSControlVertex|NURBSCurve|NURBSCurveConstPoint|NURBSCurveIntersectPoint|NURBSCurveOnSurface|NURBSCurveshape|NURBSCurveSurfaceIntersectPoint|NURBSCVCurve|NURBSCVSurface|NURBSDisplay|NURBSExtrudeSurface|NURBSFilletCurve|NURBSFilletSurface|NURBSGenericMethods|NURBSGenericMethodsWrapper|NURBSIndependentPoint|NURBSIsoCurve|NURBSLatheSurface|NURBSMirrorCurve|NURBSMirrorSurface|NURBSMultiCurveTrimSurface|NURBSNBlendSurface|NURBSObject|NURBSOffsetCurve|NURBSOffsetSurface|NURBSPoint|NURBSPointConstPoint|NURBSPointCurve|NURBSPointCurveOnSurface|NURBSPointSurface|NURBSProjectNormalCurve|NURBSProjectVectorCurve|NURBSRuledSurface|NURBSSelection|NURBSSelector|NURBSSet|NURBSSurf|NURBSSurface|NURBSSurfaceApproximation|NURBSSurfaceEdgeCurve|NURBSSurfaceNormalCurve|NURBSSurfConstPoint|NURBSSurfSurfIntersectionCurve|NURBSTexturePoint|NURBSTextureSurface|NURBSULoftSurface|NURBSUVLoftSurface|NURBSXFormCurve|NURBSXFormSurface|nvConstraint|ObjAssoc|object|Object_Color|Object_Display_Culling|Object_ID|Object_Utilities|objectIDRenderElement|ObjectParameter|objectReferenceTarget|ObjectSet|ObjExp|ObjImp|Ocean|Ocean__lume|OilTank|OkClass|Old_Point_Cache|Old_Point_CacheSpacewarpModifier|Old_PointCache|Old_PointCacheWSM|OldVertexPaint|OLEMethod|OLEObject|Omnilight|On_Off|One_Click_Particle_Flow|OneClick|OneClickSource|Opacity__base|Open_Edges|OpenEdgesClass|OpenEXR|OpenFltExport|OpenSubdiv|OpenSubdivMod|optimize|Orbaz_Mix|Oren_Nayar_Blinn|OrenNayarBlinn|Orientation_Behavior|Orientation_Constraint|OrientationBehavior|output|Output__3dsmax|Output_mParticles|OutputCustom|OutputNew|OutputPhysX|OutputStandard|OutputTest|Overlapped_UVW_Faces|OverlappedUVWFacesClass|Overlapping_Faces|Overlapping_Vertices|OverlappingFacesClass|OverlappingVerticesClass|oversampling_lens|Oversampling_Lens__physics|Paint|PaintboxStartup|PainterInterface|PaintLayerMod|PaintRenderElement|PalmTrans|Panorama_Exporter|ParamBlock2ParamBlock2|ParamBlockParamBlock|parameter|ParameterCollectorCA|ParameterEditor|PArray|ParserLoader|parti_volume|Parti_Volume__physics|Parti_Volume__physics_legacy|parti_volume_photon|Parti_Volume_Photon__physics|Parti_Volume_Photon__physicstextureMap|Particle_Age|Particle_Age__3dsmax|Particle_Bitmap|Particle_Blur__3dsmax|Particle_Cache|Particle_Face_Creator|Particle_Flow_Global_Actions|Particle_Flow_Tools_Global_Utility|Particle_Flow_Utility|Particle_MBlur|Particle_Paint|Particle_Paint_Cursor|Particle_Skinner|Particle_View|Particle_View_Global_Actions|particleBlur|ParticleCache|ParticleContainer|ParticleCreatorOSM|ParticleGroup|particleMesher|Particles|ParticleSkinnerOSM|ParticleView|Passthrough_Bump_Map__base|Paste_Skin_Weights|PasteSkinWeights|Patch_Select|PatchDeform|path|Path_Constraint|Path_Follow|Path_Follow_Behavior|Path_FollowMod|path_material|Path_Material__physics|PathCylinder|PathDeform|PathDeformSpaceWarp|PathFollowBehavior|PathName|PB2Parameter|PBomb|PBombMod|PCloud|PDAlpha|Perlin_Marble|Perlin_Marble__3dsmax|perlinMarble|PersistentIsolationData|PersistentNodeSet|Perspective_Match|PF_NotifyDep_Catcher|PF_Source|PFActionListPool|PFArrow|PFEngine|PFlow_Collision_Shape|PFNotifyDepCatcher|PFSystemPool|PhilBitmap|Phong|Phong2|Photon_Basic__base|Physical|Physical_Camera|Physical_Camera_Exposure_Control|physical_lens_dof|Physical_Lens_DOF__physics|physical_light|Physical_Light__physics|PhysicalMaterialManager|Physique|PhysX_and_APEX_Exporter|PhysX_Debug_Visualizer|PhysX_Shape_Convex|PhysX_World|PhysXBuoyancy|PhysXCollision|PhysXDrag|PhysXFlow|PhysXForce|PhysXGlue|PhysXInterCollision|PhysXModRB|PhysXPanel|PhysXPanelGlobalUtilityPlugin|PhysXPanelGUP|PhysXPanelReferenceTarget|PhysXShape|PhysXShapeConvex|PhysXShapeWSM|PhysXSolvent|PhysXSwitch|PhysXWorld|PickerControl|Pipe|PipeReferenceTarget|pivot|Pivoted|PivotPos|PivotRot|Placement_Paint|PlanarCollision|Plane|Plane_Angle|PlaneAngleManip|Planet|Planet__3dsmax|Plate_Match_MAX_R2|Plug_in_Manager|PluginMarkerForBox3|PMAlpha|pngio|Point|Point2|point3|Point3_Expression|Point3_Layer|point3_list|point3_ListDummyEntry|Point3_Mixer_Controller|Point3_Motion_Capture|Point3_Reactor|point3_script|Point3_Wire|Point3_XRef_Controller|Point3_XYZ|Point3Layer|Point3List|Point3Reactor|Point3Spring|Point3XRefCtrl|Point4|Point4_Layer|point4_list|point4_ListDummyEntry|Point4_Mixer_Controller|point4_script|Point4_Wire|Point4_XRef_Controller|Point4_XYZW|Point4Layer|Point4List|Point4XRefCtrl|Point_Cache|Point_CacheSpacewarpModifier|Point_Curve|Point_Curveshape|Point_Surf|Point_SurfGeometry|PointCache|PointCacheWSM|PointCloud|PointHelperObj|PointPacket|Poly_Select|Polygon_Counter|PolyMesh_Select|PolyMeshObject|PolymorphicGeom|PolymorphicGeomshape|PolyToolsModeling|PolyToolsPaintDeform|PolyToolsPolyDraw|PolyToolsSelect|PolyToolsShift|PolyToolsTopology|PolyToolsUVWTweak|POmniFlect|POmniFlectMod|PopCharacter|PopSegControl|PopSkinControl|PopSkinObject|Populate|Portable_Network_Graphics|Position_Constraint|Position_Expression|Position_Icon|Position_Layer|position_list|position_ListDummyEntry|Position_Manip|Position_Mixer_Controller|Position_Motion_Capture|Position_Object|Position_Reactor|position_script|Position_Value|Position_Wire|Position_XYZ|PositionLayer|PositionList|PositionManip|PositionReactor|PositionSpring|PositionValueManip|Preserve|Preset_Flow|PresetDummyOperator|PresetDummyOperIcon|PresetDummyTest|PresetFlow|PresetOperator|PresetOperIcon|PresetTest|PresetTestIcon|Primitive|Priority|Prism|ProBoolean|ProBoolObj|ProCutter|progressBar|Project_Mapping|Project_Mapping_Holder|projected|Projection|ProjectionHolderUVW|ProjectionMod|ProjectionModTypeUVW|ProMaterials__Ceramic_V2|ProMaterials__Concrete_V2|ProMaterials__Generic_V2|ProMaterials__Glazing_V2|ProMaterials__Hardwood_V2|ProMaterials__Masonry_CMU_V2|ProMaterials__Metal_V2|ProMaterials__Metallic_Paint_V2|ProMaterials__Mirror_V2|ProMaterials__Plastic_Vinyl_V2|ProMaterials__Solid_Glass_V2|ProMaterials__Stone_V2|ProMaterials__Wall_Paint_V2|ProMaterials__Water_V2|ProOptimizer|ProSound|Protractor|ProxSensor|prs|PSD_I_O|Pseudo_Color_Camera_Exposure__3dsmax|Pseudo_Color_Camera_Exposure__3dsmaxtextureMap|Pseudo_Color_Exposure_Control|pseudoColorExposureControl|Push|PushMod|PushSpaceWarp|PView_Manager|PViewManager|Pyramid|PyramidUnknown_0X00000B20|PythonHost|QTime|Quadify_Mesh|QuadMesh|quadPatch|Quadratic|QuarterRound|Quat|Quicksilver_Hardware_Renderer|radianceMap|RadioControl|Radiosity|Radiosity_Override|RagdollHelper|RagdollVisualizer|Railing|Rain|RainMaterial|Randomize_Keys|randomReferenceTarget|RandomWalk|RandomWalk_Modifier|RandomWalkSpaceWarp|Raster_Image_Packet|Raster_Image_Translator|Ray|Ray_Element|Ray_Engine|Ray_Marcher__base|RayFX|RayFXUtil|RayMeshGridIntersect|RayShadTrans|Raytrace|Raytrace_Texture_ParamBlock|RaytraceGlobalSettings|RaytraceMaterial|raytraceShadow|RCMenu|Reaction_Manager|Reaction_Master|Reaction_Set|ReactionManager|ReactionMaster|ReactionSet|Reactor_Angle_Manip|ReactorAngleManip|Rectangle|Reflect__base|Reflect_Refract|Reflect_Refract__3dsmax|Reflection|reflectionRenderElement|reflectRefract|Refract__base|Refraction|Refraction_Index__base|refractionRenderElement|RefTargContainer|RefTargMonitor|Relax|Render_Subset_of_Scene_Masking__mi|Renderable_Spline|RenderElementMgr|RenderEnvironment|RenderParticles|RendPlane|RendSpline|Repel_Behavior|RepelBehavior|Rescale_World_Units|Reservoir|Reset_XForm|Resource_Collector|Retimer|RetimerCtrl|Revit_Import|Revit_importer|rgb|RGB_Multiply|RGB_Multiply__3dsmax|RGB_Tint|RGB_to_BW|rgbMult|rgbTint|Ring_Array|Ring_Element|RingWave|Ripple|Ripplebinding|RLA|RmModel|RmModelGeometry|RolloutClass|RolloutControl|RolloutFloater|RootNodeClass|rotation|Rotation_Layer|rotation_list|rotation_ListDummyEntry|Rotation_Mixer_Controller|Rotation_Motion_Capture|Rotation_Reactor|rotation_script|Rotation_Wire|RotationLayer|RotationList|RotationReactor|RPF|RvtComponentPacket|RvtElementtPacket|RvtObjTranslator|SafeArrayWrapper|SATExport|SATImport|Save_Verification|Scalar|Scale_Expression|Scale_Layer|scale_list|scale_ListDummyEntry|Scale_Mixer_Controller|Scale_Motion_Capture|Scale_Reactor|scale_script|Scale_Test|Scale_Wire|ScaleLayer|ScaleList|ScaleParticles|ScaleReactor|ScaleXYZ|Scatter|ScatterReferenceTarget|Scene|Scene_Effect_Loader|Scene_Effect_LoaderUtilityPlugin|Scene_State|Scene_State_Manager|Scene_State_ManagerUnknown_0X0000900F|Script_Operator|Script_Test|Scripted_Behavior|ScriptedBehavior|SDeflectMod|SDeflector|Seat|section|SectionPlane|Seek_Behavior|SeekBehavior|SegTrans|Select_By_Channel|Select_Keys_by_Time|Select_Object|Select_the_Biped_for_use_as_a_retargeting_reference|SelectByChannel|SelectionSet|SelectionSetArray|Self_Illumination|Send_Out|set|Setting|sgi|Shader_List__Bump|Shader_List__Displacement|Shader_List__Environment|Shader_List__Lens|Shader_List__Output|Shader_List__Photon_Volume|Shader_List__Shadow|Shader_List__Texture|Shader_List__Volume|Shadow___Light|Shadow_Light___Side_Fade|Shadow_Transparency__base|ShadowClass|shadowMap|ShadowRenderElement|ShadowsMap|Shape_Check|Shape_Facing|Shape_Instance|Shape_Mark|ShapeControl|ShapeLibrary|ShapeMerge|ShapeStandard|SharedMoflow|SharedMoflowList|SharedMotionFlow|SharedMotionFlows|SharedMotionFlowsFloatController|Sharp_Quadratic|shaveMRGeomPasser|shaveMRHairComp|shaveMRHairGeom|shaveMRHairGeomDRA|shaveMRHairIllum|shaveMRHairShadows|shaveMRInstanceGeom|shaveMRRenderComp|shaveMRSatData|shaveMRShadowMatte|shaveMRVertexIllum|ShaveStylinIcon|Shell|Shell_Material|Shellac|ShineExp|Side_Fade|Simple__contour|Simple_Bumpmap_Map___adsk|Simple_Bumpmap_Map__adsk|Simple_Checker_Map__adsk|Simple_Checker_Map__adsktextureMap|Simple_Color_Map__adsk|Simple_Colormap_Map___adsk|Simple_Colormap_Map__adsk|Simple_Gradient_Map__adsk|Simple_Hemispherical_Map__adsk|Simple_Hemispherical_Map__adsktextureMap|Simple_Image_Map__Bitmap___adsk|Simple_Image_Map__Bitmap___adsktextureMap|Simple_Marble_Map__adsk|Simple_Marble_Map__adsktextureMap|Simple_Mtl_Ceramic__adsk|Simple_Mtl_Concrete__adsk|Simple_Mtl_Generic__adsk|Simple_Mtl_Glazing__adsk|Simple_Mtl_Hardwood__adsk|Simple_Mtl_Masonry_CMU__adsk|Simple_Mtl_Metal__adsk|Simple_Mtl_Metal_adsk_V1|Simple_Mtl_Metal_V2_adsk|Simple_Mtl_MetallicPaint__adsk|Simple_Mtl_Mirror__adsk|Simple_Mtl_Plastic_Vinyl__adsk|Simple_Mtl_Radiance__adsk|Simple_Mtl_Solid_Glass__adsk|Simple_Mtl_Stone__adsk|Simple_Mtl_Wall_Paint__adsk|Simple_Mtl_Water__adsk|Simple_Noise_Map__adsk|Simple_Noise_Map__adsktextureMap|Simple_Shape|Simple_Speckle_Map__adsk|Simple_Speckle_Map__adsktextureMap|Simple_Spline|Simple_Tile_Map__adsk|Simple_Tile_Map__adsktextureMap|Simple_Wave_Map__adsk|Simple_Wave_Map__adsktextureMap|Simple_Wood_Map__adsk|Simple_Wood_Map__adsktextureMap|SimpleFaceData|SimpleOSMToWSMMod|SimpleOSMToWSMMod2|Skeleton|SkeletonAtmospheric|SkeletonTexmap|sketchUp|Skew|Skin|Skin_Morph|Skin_Wrap|Skin_Wrap_Patch|SkinTools|SkinUtilities|SkinWrapPatch|Skylight|SLAP|Slave_Control|Slave_Point3|SlaveFloat|SlaveMatrix3|SlavePoint3|SlavePoint4|SlavePos|SlaveRotation|SlaveScale|SliceModifier|Slider_Manip|SliderControl|SliderManip|SlidingDoor|SlidingWindow|SMEGUP|Smoke|Smoke__3dsmax|smooth|SmoothModifier|smoothReferenceTarget|Snow|SoftBox|Soften|SolidColor|SOmniFlect|SOmniFlectMod|SortKey|Sound|Space_Warp_Behavior|SpaceBend|SpaceCameraMap|SpaceConform|Spacedisplace|SpaceFFDBox|SpaceFFDCyl|SpaceNoise|SpacePatchDeform|SpacePathDeform|Spaceripple|SpaceSkew|SpaceStretch|SpaceSurfDeform|SpaceTaper|SpaceTwist|Spacewave|Spawn|Speckle|Speckle__3dsmax|Specular|specularMap|specularRenderElement|speed|Speed_By_Surface|Speed_Test|Speed_Vary_Behavior|SpeedByIcon|SpeedVaryBehavior|Sphere|SphereGizmo|SphericalCollision|Spherify|spin|Spindle|SpineData2|SpineTrans2|SpinLimit|SpinnerControl|Spiral_Stair|Splat|Splat__3dsmax|Spline_IK_Control|SplineIKChain|SplineIKSolver|SplineSelect|SplineShape|Split_Amount|Split_Group|Split_Selected|Split_Source|Spray|SprayBirth|SprayCursor|SprayMaster|SprayPlacement|Spring|SpringPoint3Controller|SpringPositionController|Squeeze|SSS2_Shader|SSS2_ShadertextureMap|SSS2_Skin|SSS2_Skin_Displacement|SSS2_Surface_Shader|SSS_Fast_Material__mi|SSS_Fast_Render_Shader___mi|SSS_Fast_Render_Shader__mi|SSS_Fast_Skin_Material__mi|SSS_Fast_Skin_Material_Displace__mi|SSS_Lambert_Gamma__mi|SSS_Lightmap_Write__mi|SSS_Lightmapping_shader|SSS_Passthrough_Shader__mi|SSS_Physical_Material__mi|SSS_Physical_Shader__mi|SSS_Specular_Reflections_for_Skin__mi|Stain|Stain__lume|standard|Standard_16_bit|Standard_16_bit_Grayscale|Standard_1_bit|Standard_24_bit_LogLUV_Storage|Standard_24_bit_LogLUV_Storage_with_alpha|Standard_32_bit|Standard_32_bit_Floating_Point_Storage|Standard_32_bit_Floating_Point_Storage_Grayscale|Standard_32_bit_LogLUV_Storage|Standard_32_bit_RealPixel_Storage|Standard_64_bit_Storage|Standard_8_bit_Grayscale|Standard_8_bit_Paletted|Standard_Flow|Standardmaterial|StandardMaterialClass|StandardTextureOutput|StandardUVGen|StandardXYZGen|Standin_for_missing_MultiPass_Camera_Effect_Plugin|Star|Star_Element|Stars|StateCreator|StaticMethodInterface|StdDualVS|StdShadowGen|StepShape|STL_Check|STL_Export|STL_Import|stop|Stop_Gradually|Store_Ligth_Levels|StPathClass|Straight_Stair|Strauss|Streak_Element|Stretch|String|StringPacket|StringStream|Strokes|StructDef|Stucco|Stucco__3dsmax|subAnim|subdivide|subdivideSpacewarpModifier|Submerge|Submerge__lume|subRollout|Substance|Substance_Output|SubstanceOutput|SubstancePBWrapper|SubstanceShader|SubstanceTexture|SubstanceTexWrapper|Substituion_Offset_Transform|Substitute|Substitute_Manager|Substitute_Object|SubstituteMod|SubstituteObject|Summed_area|Sunlight|Sunlight_Daylight_Slave_Controller|Sunlight_Daylight_Slave_ControllerMatrix3Controller|Sunlight_Daylight_Slave_Intensity_Controller|Sunlight_Daylight_Slave_Intensity_ControllerFloatController|Sunlight_Slave_Controller|Sunlight_Slave_Intensity_Controller|SuperNoise|SuperSpray|surface|Surface_Approximation|Surface_Arrive_Behavior|Surface_Descriptor_Material|Surface_Follow_Behavior|Surface_Mapper|Surface_position|SurfaceArriveBehavior|SurfaceFollowBehavior|SurfDeform|SW3D_Exp|sweep|swirl|Swirl__3dsmax|Switch|symmetry|TailData2|TailTrans|Tape|Taper|Targa|Target_Area|Target_Cylinder|Target_Disc|Target_Light|Target_Linear|Target_Point|Target_Rectangle|Target_Sphere|Targetcamera|TargetDirectionallight|Targetobject|targetSpot|tcb_float|tcb_point3|tcb_point4|tcb_position|tcb_rotation|tcb_scale|Teapot|Tee|Terrain|tessellate|Test_Icon|Texmap_Clipboard|Texmaps_RaytraceMtl|TexSkyLight|text|Texture_CheckerBoard__base|Texture_Filter_Lookup__base|Texture_Lookup__base|Texture_Polka_Dot__base|Texture_Polka_Sphere__base|Texture_Remap__base|Texture_Rotate__base|Texture_Sky|Texture_Turbulence__base|Texture_Vector__base|Texture_Wave__base|TextureClass|tgaio|thePainterInterface|Thin_Wall_Refraction|Thin_Wall_Refraction__3dsmax|thinWallRefraction|This_Data|TIF|tifio|tiles|time|Timer|TimeSensor|Tint__3dsmax|to_bw|TONER|Toon_Assistant|TopBottom|topBottomMat|Torus|Torus_Knot|TouchSensor|TrackSet|TrackSetList|TrackViewPick|transform_script|Transition|Translucency|Translucency__lume|Translucent|Translucent_Shader|transmat|Transmat__physics|transmat_phen|transmat_photon|Transmat_Photon__physics|Transparency__base|TreeViewUtil|Trim_Extend|TriMesh|TriMeshGeometry|triPatch|Tube|TurboSmooth|Turn_To_gPoly|Turn_to_Mesh|Turn_to_Patch|Turn_to_Poly|TurnTogPoly|TVDummyControl|TVNode|Twist|Twist_O_Rama|Twist_O_RamaSpaceWarp|TwistoramaMod|TwistoramaWSM|Two_Sided__base|U_Type_Stair|UConstraint|UDeflector|UDeflectorMod|UndefinedClass|Unified_Image_Map__Bitmap___adsk|UnsuppliedClass|Unwrap_UVW|UOmniFlect|UOmniFlectMod|UserGeneric|Utility|Utility_BumpLookup__protein|Utility_Color_Correction_Internal__protein|Utility_ColorCorrection_Maya_Shaders__protein|Utility_ColorPassThrough__protein|Utility_Combine3Bump__protein|Utility_Elliptical_Bump__protein|Utility_Elliptical_Tex__protein|Utility_Gamma___Gain__mi|Utility_Material|Utility_Mia_Material__protein|Utility_Mix2Shader__protein|Utility_MultiMixer__protein|Utility_Switch8Shader__protein|Utility_TexLookup__protein|UTypeStair|UV_Coordinate__3dsmax|UV_Generator__3dsmax|UVW_Mapping_Add|UVW_Mapping_Clear|UVW_Mapping_Paste|UVW_Remove|UVW_Xform|Uvwmap|UVWUnwrap|V_Ray_Adv_3_20_03|V_Ray_RT_3_20_03|value|ValueRef|VDM|Vector|Vector_Displacement|Vector_Field|Vector_Field_Modifier|Vector_Map|VectorField|VectorMap|velocity|Vertex_Color|Vertex_Color__3dsmax|Vertex_Colors|Vertex_Paint_Startup_GUP|Vertex_Weld|VertexColor|VertexPaint|VertexSelection|VertexWeld|Vertical_Horizontal_Turn|Video|View2DControl|ViewportManager|ViewportManagerControl|ViewportManagerCustAttrib|Visual_MAXScript|VisualMSUtil|VIZ_Radiosity|Vol__Select|Volume__base|Volume_Fog|Volume_Fog__3dsmax|Volume_Light|Volume_Light__3dsmax|VolumeSelect|Vortex|VortexMod|VRay|VRay2SidedMtl|VRay_Exposure_Control|VRay_Lens_Effects|VRayAlpha|VRayAmbientLight|VRayAtmosphere|VRayBackground|VRayBlendMtl|VRayBmpFilter|VRayBoxFilter|VRayBPTracer|VRayBump2Normal|VRayBumpMtl|VRayBumpNormals|VRayBumpNormalsMap|VRayCarPaintMtl|VRayCaustics|VRayCausticsMap|VRayClipper|VRayColor|VRayColor2Bump|VRayCompleteMap|VRayCompTex|VRayCurvature|VRayDiffuseFilter|VRayDiffuseFilterMap|VRayDirt|VRayDisplacementMod|VRayDistanceTex|VRayDomeCamera|VRayDRBucket|VRayEdgesTex|VRayEmbree|VRayEnvironmentFog|VRayExtraTex|VRayFakeFresnelTex|VRayFastSSS|VRayFastSSS2|VRayFlakesMtl|VRayFur|VRayGlobalIllumination|VRayGlobalIlluminationMap|VRayGLSLMtl|VRayGLSLTex|VRayHairFarmGeom|VRayHairFarmMod|VRayHairInfoTex|VRayHairMtl|VRayHDRI|VRayICC|VRayIES|VRayIlluminance|VRayInstancer|VRayLanczosFilter|VRayLensEffectsMax|VRayLight|VRayLighting|VRayLightingMap|VRayLightMeter|VRayLightMtl|VRayLightSelect|VRayLut|VRayMap|VRayMatteShadow|VRayMatteShadowMap|VRayMetaball|VRayMtl|VRayMtlID|VRayMtlReflectGlossiness|VRayMtlReflectGlossinessBake|VRayMtlReflectHilightGlossiness|VRayMtlReflectHilightGlossinessBake|VRayMtlReflectIOR|VRayMtlReflectIORBake|VRayMtlRefractGlossiness|VRayMtlRefractGlossinessBake|VRayMtlSelect|VRayMtlWrapper|VRayMultiSubTex|VRayNormalMap|VRayNormals|VRayNormalsMap|VRayObjectID|VRayObjectSelect|VRayObjLight|VRayOCIO|VRayOptionRE|VRayOrnatrixGeom|VRayOrnatrixMod|VRayOSLMtl|VRayOSLTex|VRayOverrideMtl|VRayParticleColor|VRayParticleTex|VRayPhysCamAttributes|VRayPhysicalCamera|VRayPlane|VRayPointParticleMtl|VRayProxy|VRayPtex|VRayRawGlobalIllumination|VRayRawGlobalIlluminationMap|VRayRawLighting|VRayRawLightingMap|VRayRawReflection|VRayRawRefraction|VRayRawShadow|VRayRawShadowMap|VRayRawTotalLighting|VRayRawTotalLightingMap|VRayReflection|VRayReflectionFilter|VRayReflectionFilterMap|VRayRefraction|VRayRefractionFilter|VRayRenderID|VRayRT|VRayRTWrapperList|VRaySampleRate|VRaySamplerInfo|VRaySamplerInfoTex|VRayScannedMtl|VRayScatterVolume|VRaySelfIllumination|VRaySelfIlluminationMap|VRayShadow|VRayShadowMap|VRayShadowMapBake|VRayShadows|VRaySimbiontMtl|VRaySincFilter|VRaySkinMtl|VRaySky|VRaySoftbox|VRaySpecular|VRaySphere|VRaySphereFade|VRaySSS2|VRayStereoRig|VRayStereoRigFloatConstraint|VRayStereoRigSlaveControl|VRayStereoscopic|VRaySun|VRayToon|VRayTotalLighting|VRayTotalLightingMap|VRayTraceSets|VRayTriangleFilter|VRayUnclampedColor|VRayUserColor|VRayUserScalar|VRayVectorDisplBake|VRayVelocity|VRayVolumeGrid|VRayVolumeGrid_Atmosphere|VRayVolumeGrid_Light|VRayVolumeGridAtmosphere|VRayVolumeGridLight|VRayVRmatMtl|VRayWireColor|VRayWrapperList|VRayZDepth|VRBL_Export|VRImgIO|VrmlImp|VUE_File_Renderer|VzMaterialTable|VzObjectTable|VzObjectTableRecord|Wake|WakeSpaceWarp|WakeSpacewarpModifier|WalkThroughGUP|Wall|Wall_Repel_Behavior|Wall_Seek_Behavior|WalledRectangle|WallRepelBehavior|WallSeekBehavior|Wander_Behavior|WanderBehavior|Water|Water__3dsmax|Water_Surface__lume|Water_Surface_Shadow__lume|WaterCell|WaterSurface|WaterSurfaceShadow|WaterWash|WaterWash2D|Wave|Wavebinding|Waveform_Float|WaveMaster|WaveObject|waves|WeightShift|Welder|Wet|Wet_Dry_Mixer__lume|WideFlange|Width_From_Color__contour|Width_From_Light__contour|Width_From_Light_Dir__contour|Wind|Windbinding|WindowStream|Wipe|WireFloat|WirePoint3|WirePoint4|WirePosition|WireRotation|WireScale|WireTex|Wood|Wood__3dsmax|WrapAround|WrapAround__lume|WrapperList|WsmBehavior|XForm|XMLImp2|XRef_Controller|XRef_Material|XRefAtmosWrapper|XRefmaterial|XRefObject|XRefScene|xView|XYZ_Coordinate__3dsmax|XYZ_Generator__3dsmax|YUV|Z_Depth|ZRenderElement)\\b',
          name: 'support.class.core.maxscript'
        },
        {
          match:
            '(?i)\\b(atmospheric|AttributeDef|BakeElement|Base_Layer|BitmapIO|camera|colorPicker|CustAttrib|ExporterPlugin|filter|FloatController|GeometryClass|GlobalUtilityPlugin|helper|IKSolver|ImporterPlugin|light|MasterBlockController|MasterPointController|material|Matrix3Controller|MAXWrapper|MAXWrapperNonRefTarg|modifier|MorphController|MPassCamEffect|node|ParamBlock|ParamBlock2|Point3Controller|Point4Controller|PositionController|QuatController|RadiosityEffect|ReferenceMaker|ReferenceTarget|renderEffect|RenderElement|RendererClass|RotationController|ScaleController|SchematicViewUtility|Shader|Shadow|shape|SoundClass|SpacewarpModifier|SpacewarpObject|System|TexOutputClass|textureMap|ToneOperator|trackViewUtility|UtilityPlugin|UVGenClass|VideoPostFilter|XYZGenClass)(?:\\b|\\s|;|$)',
          name: 'support.class.core.superclass.maxscript'
        },
        {
          match:
            '(?i)\\b(ActionItemOverrideManager|actionMan|AnimLayerManager|assemblyMgr|AssetManager|AssignVertexColors|ATSCustomDepsOps|ATSOps|Autodesk360|AutodeskMaterialManager|autosave|AutoTangentMan|BatchProOptimizer|batchRenderMgr|BipAnalyzer|BipFilter|BipFixer|BipWorkBench|BitmapLayerManager|BitmapProxyMgr|blockMgr|BoneSys|browserMgr|BrushPresetMgr|ChannelInfo|cmdPanel|colorMan|ContainerPreferences|Containers|CreaseExplorerManager|CreaseSetManager|CUIMouseConfigManagerImplement|custAttribCollapseManager|DaylightSimulationUIOps|DaylightSimulationUtilities|DaylightSystemFactory|DaylightSystemFactory2|defaultActions|DialogMonitorOPS|DisplayManager|dragAndDrop|DwfExportPreferences|dxshadermanager|EditRenderRegion|EPolyManipGrip|ExchangeStorePackageManager|FacesOrientation|FileLinkMgr|FileResolutionManager|FlightStudio|FlightStudioExport|FlightStudioImport|FlippedUVWFaces|FlowRaytraceInterface|FrameTagManager|GhostingManager|globalDXDisplayManager|gridPrefs|Grip|Hair|HDIKSys|heightManager|HelpSystem|IBitmapPager|ICEFlowFileBirthSetup|ICEFlowShapeControl|ICEFlowSystemFactory|iDisplayGamma|IInteractionMode|IKSys|ILightRef|imerge|IMetaDataManager|InstanceMgr|InterfaceIDRegistry|iParserLoader|iray_string_options|IRTShadeTreeCompiler|IShaderManagerCreator|IsolatedVertices|IsolateSelection|ITabDialogManager|IViewportShadingMgr|LayerManager|LightingAnalysisOverlayFactory|LightingUnits|LightMeterManager|LoadSaveAnimation|LockedComponentsMan|LockedTracksMan|LuminosityUtil|MainThreadExecuteRequestManager|manip|MaterialExplorerManager|MaterialPreviewSystem|MaxNetWorkerInterface|maxOps|MaxRibbon|medit|memStreamMgr|mental_ray_Preferences|mental_ray_Public_Interface|mental_ray_string_options|menuMan|MeshInspector|MissingUVCoordinates|MouseConfigManager|msZip|MtlBrowserFilter_Manager|MultipleEdges|MXSDebugger|MxsUnitResults|NamedSelectionSetManager|NetCreateHelpers|netrender|NetworkLicenseStatusManager|NetworkManager|NitrousGraphicsManager|NodeCloneMgrTest|nodeSelectionSet|NullInterface|nvpx|nvpxConsts|objXRefMgr|objXRefs|OGSDiagnostics|OneClickRevit|openEdges|OverlappedUVWFaces|OverlappingFaces|OverlappingVertices|PaintSoftSelPresetContext|ParamCollectorOps|paramPublishMgr|paramWire|particleFlow|particleFlowUtility|PerezAllWeather|pftParticleView|PhysXPanelInterface|PlacementTool|pluginManager|pop|ProjectionIntersectorMgr|ProjectionRenderMgr|PseudoColorManager|python|qat|quadMenuSettings|radiosityMeshOps|RadiosityPreferences|reactionMgr|refhierarchy|RenderEnhancements|renderMessageManager|renderPresets|RetimerMan|RevitDBManager|RevitImportWorkflow|RingArray|rollup|s3dexporter|SceneEffectLoader|SceneExplorerManager|SceneExposureControl|SceneMissingPlugIns|SceneRadiosity|sceneStateMgr|schematicviews|simpleFaceManager|SkinUtils|SlateDragDropBridge|SME|srr_exports|statusPanel|SteeringWheelsOps|styleMgr|SubstManager|timeSlider|tmGizmos|trackSelectionSets|trackviews|tverts|UIAccessor|UtilityPanel|ValueConverter|ViewCubeOps|ViewPanelManager|ViewportButtonMgr|ViewportSSB|visualMS|visualMSCA|walkThroughOps|WorkingPivot|WorkspaceManager|xViewChecker)\\b(?=\\s|;|$)',
          name: 'support.class.core.interface.maxscript'
        },
        {
          match:
            '(?i)\\b(cameras|geometry|helpers|lights|objects|selection|shapes|spacewarps|systems)\\b(?!\\.|:)',
          name: 'support.variable.objectset.maxscript'
        },
        {
          match:
            '(?i)\\b(CAT_ParamBlock|CAT_UIItem|CAT_UIItem2|CINCfg|CombustionWorkSpace|ControlValLookup_struct|CrowdAreaBrushSizePathSpinnerCallback|CrowdAreaBrushSizeSpinnerCallback|CrowdAreaCircleSidesSpinnerCallback|cwschannel|cwscomposite|cwskey|cwsobject|cwsoperator|DebugVisualizerGroup_struct|DebugVisualizerParam_struct|Disp_Angle_Spinner|Disp_Distance_Spinner|Disp_Edge_Spinner|Disp_Steps_Spinner|DistanceSelectSpinnerCallback|DontShowAgainDialog|DotLoopSpinnerCallback|Edge_Crease_Spinner|Edge_Weight_Spinner|Filters|IMouseDragger|InsertVSpinnerCallback|KeyVal_struct|lightCreationToolStr|ListViewOps|Main_Ribbon|mcrUtils|meditUtilities|MultiEdit_ControlInfo_struct|NormalSelectSpinnerCallback|NumericSpinnerCallback|NumFramesSpinnerCallback|NURMS_Cage_ColorSwatch|NURMS_CageSelected_ColorSwatch|NURMS_Iterations_Spinner|NURMS_RenderIterations_Spinner|NURMS_RenderSmoothness_Spinner|NURMS_Smoothness_Spinner|ObjectPaintFillCountSpinnerCallback|ObjectPaintMotionSpinnerCallback|ObjectPaintOffsetSpinnerCallback|ObjectPaintRotXSpinnerCallback|ObjectPaintRotYSpinnerCallback|ObjectPaintRotZSpinnerCallback|ObjectPaintScaleRampXEndSpinnerCallback|ObjectPaintScaleRampXStartSpinnerCallback|ObjectPaintScaleRampYEndSpinnerCallback|ObjectPaintScaleRampYStartSpinnerCallback|ObjectPaintScaleRampZEndSpinnerCallback|ObjectPaintScaleRampZStartSpinnerCallback|ObjectPaintScaleRandomXMaxSpinnerCallback|ObjectPaintScaleRandomXMinSpinnerCallback|ObjectPaintScaleRandomYMaxSpinnerCallback|ObjectPaintScaleRandomYMinSpinnerCallback|ObjectPaintScaleRandomZMaxSpinnerCallback|ObjectPaintScaleRandomZMinSpinnerCallback|ObjectPaintScaleXSpinnerCallback|ObjectPaintScaleYSpinnerCallback|ObjectPaintScaleZSpinnerCallback|ObjectPaintScatterUSpinnerCallback|ObjectPaintScatterVSpinnerCallback|ObjectPaintScatterWSpinnerCallback|ObjectPaintSpacingSpinnerCallback|ObjectPaintStruct|Paint_PaintSize_Spinner|Paint_PaintStrength_Spinner|Paint_PaintValue_Spinner|PDBranchSpinnerCallback|PDDistanceSpinnerCallback|PDOffsetSpinnerCallback|PDSolveSpinnerCallback|PerspectiveGrowSpinnerCallback|PerspectiveSelectSpinnerCallback|PolyBoostStruct|PolyToolsUIstruct|ProceduralContentOps|PromptForNameHandler_struct|PSConformSpinnerCallback|PSFalloffSpinnerCallback|PSFullStrengthSpinnerCallback|PSStrengthSpinnerCallback|px_multiedit_methods_struct|pxNodeKeys|PxRolloutInfo_struct|RandomConnectSpinnerCallback|RandomNumberSpinnerCallback|RandomPercentSpinnerCallback|RElement|Ribbon_Modeling|rolloutCreator|RTT_methods_struct|s_rc2mxs|SculptNoiseIterationsSpinnerCallback|SculptNoiseScaleSpinnerCallback|SculptNoiseSeedSpinnerCallback|SculptOffsetSpinnerCallback|SculptSizeSpinnerCallback|SculptStrengthSpinnerCallback|SeatFemalePctSpinnerCallback|SelectByAngleSpinnerCallback|SetFlowSpeedSpinnerCallback|SetFlowSpinnerCallback|SetVertexAlpha_Spinner|SetVertexColorColor_ColorSwatch|SetVertexColorIllum_ColorSwatch|Snaps|SortedArray_Struct|SS_Bubble_Spinner|SS_EdgeDist_Spinner|SS_FallOff_Spinner|SS_PaintSize_Spinner|SS_PaintStrength_Spinner|SS_PaintValue_Spinner|SS_Pinch_Spinner|SurfaceSelectSpinnerCallback|sxmlIO|TreeViewWraps|uvwManipUtils|UVWTweakFalloffSpinnerCallback|UVWTweakFullStrengthSpinnerCallback|UVWTweakSpinnerCallback|UVWTweakStrengthSpinnerCallback|Vertex_Crease_Spinner|Vertex_Weight_Spinner|VertexSelect_B_Spinner|VertexSelect_ColorSwatch|VertexSelect_G_Spinner|VertexSelect_R_Spinner|VFB_methods_struct|VFB_ViewportInfo|VFB_ViewportInfoEntry|VRSceneExportInfo)\\b(?=\\s|;|$)',
          name: 'support.class.structdef.maxscript'
        },
        {
          match:
            '(?i)\\b(autoBackup|BezierDefaultParams|biped|cui|displayColor|gw|hideByCategory|keyboard|logsystem|MatEditor|mocap|mouse|options|pathConfig|preferences|renderers|scanlineRender|snapMode|sysInfo|TCBDefaultParams|themixer|timeConfiguration|toolMode|trackbar|units|viewport|WAVsound)\\b(?=\\s|;|$)',
          name: 'support.class.system.structdef.maxscript'
        },
        {
          match:
            '(?i)\\b(ActionCreateFlow|ActionEditFlow|ActionExtendFlow|ActionIdleAddMode|ActionIdleSubtractMode|AddConstraint|AddListController|AddMod|AddPModObjects|APEXSaveFBX|appendMenuItem|ApplyOperation|arrBitMapTxtPaths|AssignConstraintSelection|AssignControllerSelection|autoinstallVRayUI|BakeSetupController|bakeSetupController_AbsoluteMode|bakeSetupController_AdditiveMode|BakeUnsetupController|bakeUnsetupController_DontRevert|bakeUnsetupController_Revert|begin_asset|boolToInteger|CallbackFn1|camerasToVRay|CaptureAnimation|CAT_OnMaxShutdown|CAT_OnMaxStartup|CATCollapseLayers|CATImportBip|CATImportBVH|CATImportHTR|CATParentSetupMode|checkBakedPositionName|checkBakedRotationName|clearArrBitMapTxtPaths|close_tag|closeVismatTag|ConstrFilterFn|convertB2HDRI_img2tiledexr|convertB2HDRI_NodeMtl|convertB2HDRI_SubMtl|convertB2HDRI_SubTex|convertB2HDRI_Texmap|convertCoronaTexToVRayTex|convertFrom_ArchAndDesign|convertFrom_Architectural|convertFrom_CoronaLightMtl|convertFrom_CoronaMtl|convertFrom_CoronaRaySwitchMtl|convertFrom_CoronaShadowCatcherMtl|convertFrom_CoronaVolumeMtl|convertFrom_fR_Advanced|convertFrom_fR_Glass|convertFrom_fR_Metal|convertFrom_Raytrace|convertFrom_Standard|convertFromBlend|convertFromDoubleSided|convertFromShellac|convertToVRay|coronaProxiesToVray|createBmpFilesInVismatFolder|CreateCameraFromView|CreateDefaultDaylightSimulationMaterial|createPointCloudMaterial|CreateValidDaylightSimulationLightCallback|createVRayMtl|CrowdAreaEndTools|CrowdAreaToolToggle|cwsWarn|debugVisualizerApexClothingAutoUI|debugVisualizerAutoUI|debugVisualizerClothingAutoUI|DoesSelectedContainInterface|dumpBitmapBuffer|dumpBRDFDiffuse|dumpBRDFVRayMtl|dumpColorTexHandlerParam|dumpComponentTransformParameterForMapCoords|dumpComponentTransformParameterForPoint3MapCoords|dumpMap|dumpMaterials|dumpMtlASGVIS|dumpMtlSingleBRDF|dumpMultiMtl|dumpParam|dumpSelectedMaterial|dumpSubMaterial|dumpTexAColorFromColor|dumpTexBitmap|dumpTexCellular|dumpTexChecker|dumpTexColorCorrection|dumpTexCombineColor|dumpTexCompMax|dumpTexFalloff|dumpTexMix|dumpTexMulti|dumpTexNoiseMax|dumpTexSpeckle|dumpTexTiles|dumpTextureParams|dumpTextureParamsListVals|dumpTexVRayHDRI|dumpUVWGenChannel|dumpUVWGenChannelForMap|dumpUVWGenObjectForMap|dumpUVWGenObjectScale|dumpVRay2SidedMtl|dumpVRayBlendMtl|dumpVRayBRDFLayered|dumpVRayMtl|dumpVRayMtlTexmaps|dumpVRayMtlWrapper|DVSPxForm|DVSPxFormCam|DYNFUNbakeXFCCtoPRS|DYNFUNcolorChangedCallback|DYNFUNdisplayEditText|DYNFUNdisplayProgressPanel|DYNFUNdnColor|DYNFUNdnTreeViewStyle|DYNFUNdoesFolderExist|DYNFUNdynamiteInitialise|DYNFUNfilePostMergeCallback|DYNFUNfilePostOpenCallback|DYNFUNfilePostSaveCallback|DYNFUNgetCivilViewCategorizedNodeArrays|DYNFUNgetcountrykits|DYNFUNgetDaylightMembers|DYNFUNgetfiles|DYNFUNiniBaseClear|DYNFUNiniReadFromFolder|DYNFUNinitializeCivilView|DYNFUNisMentalRayProductionRenderer|DYNFUNlistLibraryMaps|DYNFUNloadMapPaths|DYNFUNloadVSPnodeCallBacks|DYNFUNloadVSPsceneCallBacks|DYNFUNmakeCivilViewSurfaces|DYNFUNmakeCivilViewVehicles|DYNFUNmakeResourceKitStructure|DYNFUNmaxBakingControl|DYNFUNmaxXForm|DYNFUNmaxXformCam|DYNFUNnodeCreatedCallback|DYNFUNnodeRenamedCallback|DYNFUNnormalizeMenuName|DYNFUNpostImportCallback|DYNFUNpostRendererChangeCallback|DYNFUNpreSystemShutdownCallback|DYNFUNrebuildExplorerTreeNodes|DYNFUNremoveCivilViewFromMainMenu|DYNFUNremoveCSProllouts|DYNFUNresetEnvironmentMap|DYNFUNresetFogParams|DYNFUNresetXformControllers|DYNFUNsceneRedoCallback|DYNFUNsceneUndoCallback|DYNFUNselectedNodesPostDeleteCallback|DYNFUNselectedNodesPreDeleteCallback|DYNFUNsetCountryPaths|DYNFUNsetUpMaterialLibraries|DYNFUNsetupRootNode|DYNFUNsystemPostNewCallback|DYNFUNsystemPostResetCallback|DYNFUNupdateExplorer|DYNFUNupperCase|DYNFUNviewportSetup|DYNFUNvizXform|DYNFUNvizXformCam|end_asset|enumVRayStandaloneInstalls|execVRay2SidedMtlParam|execVRayMtlParam|ExportMesh|findFirstSeparatorIdx|findLastSeparatorIdx|findMenuItem|findMenuItemIdx|findobject|FindPMod|FixInvalidMaterialsForDaylightSimulation|GenerateControlChangedFn|GenerateControlEventHandler|GenerateControlSection|GenerateFullRolloutString|GenerateGroupChangedFunctions|GenerateLoadValuesFn|GenerateSaveValuesFn|GenerateUndoFunction|GeomFilterFn|GetCATNodeGroup|GetCATParent|getCINCfg|getData_InstProxy_byOrig|getMaxReleaseAsString|getOrigCoronaProxy|getParamType|GetPersWorldPosForMouse|GetPModObjectNames|GetSelectedCATParent|GetSelectedSubAnim|getTextureParamList|getVRayInstallPath|gPxRBKeys|hasMacroRecorderContext|help|InchesToSystemScale|initGroupMembers|InsertMenuItem|insertSubmenuItem|IsCATEntity|IsCwsImgType|IsFloatController|IsLayerControl|isMenuItemExist|IsSelectionSubAnimClassOf|IsValidControllerSelection|isVRayCompatible|KindOfRenderElement|LightCreationZHeight|lightsToVRay|listNormalBumpFeatures|loadCUIScheme|make_acolortexture_parameter_xml|make_bool_parameter_xml|make_color_parameter_xml|make_list_parameter_xml|make_parameter_xml|MaxPlusINodeFromNode|mtlsToVRay|new_mtls|normalizeColor|open_tag|openVismatTag|orig_mtls|popSkinCharSelected|px_add_rigidbody|PX_CreateBoundingBox|PX_CreateBoundingCapsule|PX_CreateBoundingConvex|PX_CreateBoundingConvexForNodes|PX_CreateBoundingSphere|PX_CreateConvexHullFromPoints|px_filePostOpen|px_filePreOpen|px_filePreSave|px_selectionChanged|px_selectionChangedDirty|px_stopSimulationForEditing|px_syncRagdollList|px_systemPostNew|px_systemPostReset|px_systemPreNew|px_systemPreReset|px_systemPreShutdown|px_UpdateSelectedRigidBodyLists|PxAddAllPhysXObjects|PxAddApexClothingMod|PxAddApexClothingToNodes|PxAddApexSBClothingMod|PxAddApexSBClothingToNodes|PxAddD6Joint|PxAddModifierToTrash|PxAddSpring|PxApexModifierPostAdded|PxApexModifierPreAdded|PxApexModifierPreDelete|PxApexModsToDelete|PxApexModsToDeleteForce|pxBake|pxBakeAll|pxBakeClothing|pxBakeNodes|pxBakeNodesUndoOn|pxBakeSelection|pxBakeSelectionUndoOn|PxCalcJointLimits|PxCalculateConstraintPose|PxCaptureTransform|PxCheckFileVersion|PxCheckOldVersionMenus|PxCheckSDK3|PxClearSavedRBKeys|PxCloseAllUI|PxCloseMenu|PxClothModifierPostAdded|PxClothModifierPreAdded|PxClothModifierPreDelete|PxClothNodes|PxConstraintDefaultName|PxConvertOldJoint|PxConvertOldRB|PxCreateAs|PxCreateAsCloth|PxCreateAsDynamic|PxCreateAsKinetic|PxCreateAsStatic|PxCreateBox|PxCreateCapsule|PxCreateConstraint|PxCreateConstraintAt|PxCreateConstraintForNodes|PxCreateConstrintBallSocket|PxCreateConstrintGear|PxCreateConstrintHinge|PxCreateConstrintRigid|PxCreateConstrintSlide|PxCreateConstrintTwist|PxCreateConstrintUniv|PxCreateDynamicRagdoll|PxCreateDynamicRagdollHelper|PxCreateKinematicRagdoll|PxCreateKinematicRagdollHelper|PxCreatePhysXScene|PxCreateSphere|pxDebugVisualizerUndo|PxDeleteBrokenRBs|PxDeleteRB|PxDynamicRBNodes|PxEmptyModifierTrash|PxExistFileProperty|PxExporterLoadLastParameters|PxExporterSaveLastParameters|PxExportFBX|PxExportPxProj|PxExportPxProj_Prompting|PxExportTemplate|PxFindApexGraphicalMod|PxFindApexLODMod|PxFindGeometryChange|PxForceCreatePhysXScene|PxGenerateApexClothingDebugVisualizerRollout|PxGeneratePhysXClothingDebugVisualizerRollout|PxGetAllChildrenNodes|PxGetCurrentNode|PxGetCurrentNodeType|PxGetDisplayUnitFactor|PxGetFileProperty|PxGetGroupNodes|PxGetGroupNodesCount|PxGetIniFilename|PxGetMeterToUnit|PxGetModCloth|PxGetModClothByClass|PxGetModRB|PxGetModRBByClass|PxGetNodeByHandle|PxGetNodeHandle|PxGetNodePivot|PxGetNodePosition|PxGetNodeType|PxGetNodeUserProp|PxGetOldRagdoll|PxGetPhysXPanelTab|PxGetRagdollHelper|PxGetRagdollHelperNode|PxGetRagdollHelperRootBone|PxGetRagdollRootBone|PxGetRootParent|PxGetSelectedDummyHelpers|PxGetSelectedDummyHelpersCount|PxGetSelectedHelpers|PxGetSelectedHelpersCount|PxGetSelectedNodes|PxGetSelectionCount|PxGetSimPlaybackSetting|PxGetSingleNodes|PxGetSystemCommandMode|PxGrabPositions|PxGrabRotations|pxGroupNodes|PxImportAllHelperMeshes|PxImportMesh|PxImportRagdollHelperMesh|PxImportTemplate|PxInitGlobalParamsForNewFile|PxInitializePlugin|PxInitJointParams|PxIsApexGraphicalMod|PxIsBakeable|PxIsBakeableAtTime|PxIsBaked|PxIsBone|PxIsBuildInShape|PxIsClothingModifierSelected|PxIsClothMod|PxIsDynamicRB|PxIsKinematicRB|PxIsPhysicsObject|PxIsPhysXMod|PxIsRB|PxIsRigidBody|PxIsSimulationRunning|PxIsStaticRB|PxKeepGlobalParametersUnchanged|PxKeepPhysicsValuesUnchanged|PxKinematicRBNodes|PxListenGeneralEvents|PxLoadFileProperties|PxLoadGlobalParam|PxLoadRolloutInfoList|PxLoadSavedPose|PxMakeCloth|PxMakeDynamicRB|PxMakeKinematicRB|PxMakeStaticRB|PxMaxIsEditing|PxModRBCloneHappend|PxModRBMirrorHappened|PxModRBPostAdded|PxModRBPreAdded|PxModRBPreDelete|PxNeedSolveCompatibility|PxNeedToBake|PxNodeIsConvex|PxPanelClose|PxPanelUndo|PxParseModRB|PxPauseSimulation|PxPotentialRagdoll|PxPotentialRagdollHelper|PxPrepareConstraintName|PxPrintExtraShapes|PxRagdollList|PxRagFilePostOpen|PxRBGetVolume|PxReadMaxUnit|PxReadSystemUnit|PxRebuildSelectedClothing|PxRecordExistingPhysXMod|PxRefreshD6PropertyPanels|PxRefreshPhysXToolPanel|PxRefreshRBPropertyPanels|PxRemoveCloth|PxRemoveClothMod|PxRemoveFormerPhysXMod|pxRemoveInstances|PxRemoveOldRagdoll|PxRemovePhysXMods|PxRemoveRagdollByNode|PxRemoveRagdollHelper|PxResetGlobalParams|PxResetPhysXScene|PxResetRBKeys|pxRestoreBake|PxRunSimulation|PxSaveGlobalParams|PxSaveRBKeys|PxSaveRolloutInfoList|PxScaleHelperSize|PxSelectBoneDialogList|PxSelectBoneDialogResult|PxSelectBonesDialog|pxSelectedConstraints|pxSelectedDummyHelpers|pxSelectedHelpers|pxSelectedNodes|pxSelectedRigidBodyMods|pxSelectedRigidBodyObjs|pxSelectedShapes|PxSeletionCanCreateCloth|PxSeletionCanCreateRB|PxSetCCDMotionThreshold|PxSetContactDistanceFromRestDepth|PxSetGeometryScale|PxSetGlobalParamsFromPhysXPanelInterface|PxSetPhysXPanelInterfaceFromGlobalParams|PxSetProperty|PxSetRBHullNodeColor|PxSetSimPlaybackSetting|PxSetSystemCommandMode|pxShapeHandles|PxShowAbout|PxShowExportOptions|PxShowExportOptionsDialog|PxShowPhysicsPanel|PxShowPluginBuild|PxShowViewer|PxSimClock|PxSimulateNFrames|PxSimulateOneFrame|pxSingleNodes|PxSolveCompatibilityIssues|PxStartPlugin|PxStaticRBNodes|PxStepSimulation|PxStopSecond|PxStopSimulation|PxSwitchSDK|PxSwitchSDKSilent|PxSystemUnitChangeFactor|pxUnbake|PxUnitEnumToType|PxUnitsChange|PxUnitTypeToEnum|PxUpdateGravity|PxUpdateGroundPlane|PxUpdateModRBMassForGeometryChange|PxUpdatePhysXParameters|PxUpdateRolloutInfoList|PxUseHardwareScene|PxUseMultiThread|registerMenuItems|registerVRayEnhancedMenuBarItems|registerVRayMenuBarItems|registerVRayMenus|registerVRayQuadMenuItems|RElements2cws|replaceCameraNode|replaceCoronaLightNodeVRayLight|replaceCoronaSunLightNodeVRayLight|replaceLight_mr_Sun|replaceLightNode|replaceLightNodeVRIESLight|replaceMtlNode|SaveQuadClr|selectionChangeFn|SelectionHasCATParent|SetActiveController|setupVismatConverter|shadowsNode|shadowsToVRay|ShapeFilterFn|show|showMtlSlotInSME|sqr|SurfaceFilterFn|TurnOnMacroRecorderContext|TurnToSeq|typeListGIQualityInterpolatedValues|unregisterVRayEnhancedMenuBarItems|unregisterVRayMenuBarItems|unregisterVRayMenus|unregisterVRayQuadMenuItems|UpdatePointCloudMaterial|updateVRayMenus|ValidateExposureControlForLight|value_to_xml|value_to_xml_string|vrayBitmap2VRayHDRI|vraymax_color_to_xml_string|vraymax_colorwithalpha_to_xml_string|vrayToolBarMenuRegistration|vrscene_renderExport|warningToSelectGeometry|write_cws_channel|write_cws_cineonConverter|write_cws_composite|write_cws_composite_settings|write_cws_connect|write_cws_eof|write_cws_footage|write_cws_head|write_cws_object|write_cws_operator|write_cws_parenting)\\b(?=\\s|;|$|\\()',
          name: 'support.function.maxscript'
        },
        {
          match:
            '(?i)\\b(__rcCounter|_meditMaterialsBeforeReset|_SetProjectFolder_macro_option_newFolder|_SetProjectFolder_macro_option_promptUser|animposeini|applymaxIKCA|BoneAdjustmentsFloater|BoneAdjustmentsFloater_FinToolsRollout|BoneAdjustmentsFloater_updateOPRFlag|BoneAdjustmentsFloater_updateTRFlag|CamPerspCorrect|CAT_CurrentDef|CAT_Debug|CAT_Debug2|CAT_DEF|CAT_EditUIRoll|CAT_EditUIRoll_Height|CAT_FinishRoll|CAT_FinishRoll_Open|CAT_Floater|CAT_OpenDialog|CAT_Param_Remap|CAT_ParamBlock_Array|CAT_Reset|CAT_SelChange|CAT_SetTargetObject|CAT_Source|CAT_TargetObject|CAT_TestRO_Open|CAT_TestRoll|CAT_UI_POS|CAT_UI_Size|CAT_UIItems_Array|CAT_UINum|CAT_UIOptionRO_Open|CharacterAssembly|Civil_Structure|CivilView_Alignment|CivilView_Pipe|collapseController2Controller|collapseController2EulerController|color_lookup|ColorPaletteRollout|conv_area_lights|convertB2HDRI_ConvertCompression|convertB2HDRI_ConverterErrors|convertB2HDRI_ConverterPath|convertB2HDRI_ConvertEXR|convertB2HDRI_ConvertHalf|convertB2HDRI_ConvertLinear|convertB2HDRI_ConvertTilesize|convertB2HDRI_Rollout|convertB2HDRI_TEXRConverterRollout|cws_ops_array|default_color_names|default_color_values|DisplayOKCancelDontShowAgainDialogDataInstance|dSightRender|DYNalignmentParams|DYNamite|DYNatmosphere|DYNattachParams|DYNbinPath|DynBuilding|DYNbuildingObjects|DYNbuildingParams|DYNbuildShape|DYNc3dImport|DYNcamParams|DYNchainageText|DYNcivilViewCategoryCount|DYNcivilViewProductName|DYNcivilViewStarted|DYNcivilViewVersion|DYNcountryKitFolderSeedName|DYNcountryPath|DYNcreatable|DYNcveImport|DYNdnColorClass|DYNdnSmImageList|DYNdotNetCountryINIbase|DYNdotNetPrivateINIbase|DYNdotNetProjectINIbase|DYNdrapeSpline|DYNdsimImport|DynDSIMmenuMain|DynDSIMmenuTools|DYNdsimSubstitute|DYNdsimSurface|DynDVSPmenuImport|DynDVSPmenuMain|DYNdxfImport|DYNexposure|DYNfeatureInterp|DYNfileSelector|DYNfogName|DYNfolderNameBuildingMaps|DYNfolderNameCameras|DYNfolderNameFurniture|DYNfolderNameFurnitureMaps|DYNfolderNameMarkingMaps|DYNfolderNameMatLibs|DYNfolderNameObjLibs|DYNfolderNamePreviews|DYNfolderNamePrimitives|DYNfolderNameRailMaps|DYNfolderNameSignMaps|DYNfolderNameSigns|DYNfolderNameSurfaceMaps|DYNfolderNameTreeMaps|DYNfolderNameTrees|DYNfolderNameVehicleMaps|DYNfolderNameVehicles|DYNfolderSeperator|DYNforceManualStart|DYNforestParams|DYNgantryParams|DYNglobalMapsPath|DYNglobalShiftCustomAttributes|DYNgRailObjects|DYNgRailParams|DYNiconsPath|DYNimageTypes|DYNimportCatsArray|DYNimportMan|DYNimportSuperClassArray|DYNINIallowMultMarks|DYNINIallowMultRails|DYNINIautoStart|DYNINIchainageLoop|DYNINIcomPanel|DYNINIcountry|DYNINIdisableGrid|DYNINIdockExplorer|DYNINIenableFog|DYNINIendMarkerColor|DYNINIexplorerWidth|DYNINIfaceSmoothing|DYNINIframeRate|DYNINIgreyedOutColor|DYNINIhideDiagonals|DYNINIhighlightColor|DYNINImarkerSize|DYNINImarkingColor|DYNINImarkingMaxDistance|DYNINImarkingSpacing|DYNINImarkingThreshold|DYNINImatIDlimit|DYNINImemPrecent|DYNINIminHeapSize|DYNINIoverhangCorrect|DYNINIprivateKit|DYNINIprivateKitPath|DYNINIprojectKit|DYNINIprojectKitPath|DYNINIreadMethod|DYNINIresetMaxMenu|DYNINIretainOPSmat|DYNINIrotateWheels|DYNINIrWorldX|DYNINIrWorldY|DYNINIrWorldZ|DYNINIshadowSaver|DYNINIshowDefaultsWarning|DYNINIshowIcons|DYNINIshowResourceKitWarning|DYNINIshowUnitsWarning|DYNINIspeedValue|DYNINIstartMarkerColor|DYNINIstepThreshold|DYNINIt|DYNINItreeVariation|DYNINItype|DYNINIungpMatChan|DYNINIvehNr|DYNINIvehTypeName|DYNINIworldX|DYNINIworldY|DYNINIworldZ|DYNlampParams|DYNlightList|DYNmainFloater|DYNmarkingParams|DYNmatEditor|DYNmatNameBuildings|DYNmatNameEnvironment|DYNmatNameMarkings|DYNmatNameObjects|DYNmatNameRails|DYNmatNameSurfaces|DYNmatNameTrees|DYNmatNameVehicles|DYNmaxMajorVersionString|DYNmessageRollout|DYNmrFinalGather|DYNmrPhysicalSky|DYNmrSkyLight|DYNmrSunLight|DYNobjectPlacer|DYNobjHandles|DYNoldMatNameBuildings|DYNoldMatNameEnvironment|DYNoldMatNameMarkings|DYNoldMatNameObjects|DYNoldMatNameRails|DYNoldMatNameSurfaces|DYNoldMatNameTrees|DYNoldMatNameVehicles|DYNpathObjectList|DYNplacedMan|DYNpNetworkParams|DYNpointBasedPlace|DYNpositionControl|DYNpreferences|DYNprimitiveParams|DYNproperties|DYNpSystemParams|DYNqparImport|DYNremovePOLrefreshEnable|DYNresourceList|DYNresourceMan|DYNroadMarkings|DYNrootNodeStore|DYNscriptsPath|DYNshapeParams|DYNsightTool|DYNsightToolFont|DYNsignalParams|DYNsignParams|DYNskyDome|DYNskyMapPath|DYNsosParams|DYNsplashPanel|DYNsplineMapper|DYNstdLight|DYNstringBOSfolderName|DYNstringFISfolderName|DYNstringOPSfolderName|DYNstringRMSfolderName|DYNstringROSfolderName|DYNstringSOSfolderName|DYNsunLight|DYNsurfaceParams|DYNsweptObjects|DYNtempPath|DYNtreeParams|DYNuiResourcesCivilViewGlobal|DYNuiResourcesCvExplorer|DYNuiResourcesObjClasses|DYNuserInterface|DYNvehicleParams|DYNwheelRotationControllerAttributes|DYNworldStatistics|EC_HelperOBJ|EC_OBJ|EC_SplineOBJ|EC_TargetOBJ|EmptyModifier|EmptyModifier_Old|execParam_current_object|floatlt|g_dragger|g_suspendCharacterRedraw|gEnableDebug|gPhysXMenuRunSimItem|gPxAnimationLoopOn|gPxBounceMinSpeedAutomatic|gPxButtonWidth|gPxCCDMinSpeedAutomatic|gPxClothPropertyPanel|gPxCreateConstraintBallSocketMode|gPxCreateConstraintGearMode|gPxCreateConstraintHingeMode|gPxCreateConstraintRigidMode|gPxCreateConstraintSlideMode|gPxCreateConstraintTwistMode|gPxCreateConstraintTypeFlag|gPxCreateConstraintUnivMode|gPxCreatingCount|gPxCreationChild|gPxCreationHelper|gPxCreationParent|gPxD6PropertyPanel|gPxDestructionDamage|gPxDestructionMomentum|gPxDestructionRadius|gPxDialogWidth|gPxEitterPropertyPanel|gPxFirstPhysXFrame|gPxFluidsPropertyPanel|gPxGenerateShapePerElement|gPxGeometryScale|gPxGroundHeight|gPxIconOffset|gPxJointProjAngular|gPxJointProjLinear|gPxLoopAnimation|gPxMaxDialogHight|gPxOnLastFrame|gPxRBPropertyPanel|gPxReasonBake|gPxReasonExport|gPxReasonModifierCancel|gPxReasonModifierNone|gPxReasonModifierSelected|gPxReasonPlayback|gPxReasonSettle|gPxReasonUnbake|gPxSimClockLastTime|gPxSimulationDisabled|gPxSimulationPreparing|gPxSimulationRewind|gPxSimulationRuning|gPxSleepThresholdsAutomatic|gPxSoftBodyPropertyPanel|gPxUIOffset|gPxUseFirstFrame|gPxUseGround|gPxUseHardwareScene|gPxUseMultiThread|gPxVolumeUnitChange|gSimClock|gTextureBakeDialog|gViewerClock|gXmlIO|IdleAreaObj|KeyValueUtility|lastSelectedPreset|Light_RollAngle_Manipulator|LightCreationTool|listLanguageIdxs|listOfMAPCoordsStyles|LLister|LListerYOffset|lvops|mainMenuBar_Rendering|mappinglayerData|menuName_Create|menuName_Create_Cameras|menuName_Create_Helpers|menuName_Create_Lights|menuName_Enhanced_LightingCameras|menuName_Enhanced_LightingCameras_Tools|menuName_Enhanced_Materials|menuName_Enhanced_Materials_Tools|menuName_Enhanced_Objects|menuName_Enhanced_Rendering_Preview|menuName_Modifiers|menuName_Tools|menuName_Tools_LightLister|miAreaLight|miAreaLightomni|namedSelSetsData|NodeJoeAttrDef|NodeJoeNodeAttrDef|NodeJoeNodeViewAttrDef|nvBox|nvCapsule|nvpx_post_system_startup|nvpxText|nvRagdoll|nvSphere|nvSpring|ObjectPaint|ObjectPaintNodeDialog|objPos|param_lookup|PBo_BSmain|PBo_getmain|PBo_NMmain|PBo_Pickobject|PBo_RCmain|PBo_RDmain|PBo_Simimain|PBPTmainroll|PBtexprog|PBViewCanvas|PHYSX_AUTODESK_VER|physXpaneldata|PolyBoost|PolyToolsUI|presetIsLoaded|PromptForNameHandler|PromptForNameRollout|PT_topomap|PT_topomapM|px_current_version|px_d6Panel_advanced|px_d6Panel_connection|px_d6Panel_spring|px_d6Panel_swingtwist|px_d6Panel_translation|PX_DOF_FREE|PX_DOF_LIMITED|PX_DOF_LOCKED|px_exporter_clothing|px_exporter_destruction|PX_EXPORTER_EXPORTING|px_exporter_general|px_exporter_options|px_exporter_scene_settings|PX_EXPORTER_SELECTED_ONLY|PX_EXPORTER_VISIBLE_ONLY|PX_LASTFRAME_CONTINUE|PX_LASTFRAME_LOOP|PX_LASTFRAME_PAUSE|PX_MAXVERSION_2009|PX_MAXVERSION_2010|PX_MAXVERSION_2011|PX_MAXVERSION_2012|PX_MESHTYPE_BOX|PX_MESHTYPE_CAPSULE|PX_MESHTYPE_COMPOSITE|PX_MESHTYPE_CONVEX|PX_MESHTYPE_CUSTOM|PX_MESHTYPE_ORIGINAL|PX_MESHTYPE_SPHERE|px_multiedit_methods|px_panel_advancedSettings|px_panel_debugVisualizer|px_panel_debugVisualizerApexClothing|px_panel_debugVisualizerClothing|px_panel_engine|px_panel_globalParameters|px_panel_multiedit_advanced|px_panel_multiedit_boxParams|px_panel_multiedit_capsuleParams|px_panel_multiedit_compositeParams|px_panel_multiedit_constraints_advanced|px_panel_multiedit_constraints_general|px_panel_multiedit_constraints_spring|px_panel_multiedit_constraints_swingTwistLimits|px_panel_multiedit_constraints_translationLimits|px_panel_multiedit_convexParams|px_panel_multiedit_customParams|px_panel_multiedit_forces|px_panel_multiedit_material|px_panel_multiedit_materialList|px_panel_multiedit_mesh|px_panel_multiedit_originalParams|px_panel_multiedit_properties|px_panel_multiedit_selection|px_panel_multiedit_sphereParams|px_panel_rigidBodyDisplay|px_panel_simulation|px_panel_tools_destruction|px_panel_tools_simulation|px_panel_tools_utilities|PX_PHYSBAKED_DYNAMIC|PX_PHYSBAKED_KINEMATIC|PX_PHYSTYPE_CLOTH|PX_PHYSTYPE_CLOTHING|PX_PHYSTYPE_DESTRUCTION|PX_PHYSTYPE_DYNAMIC|PX_PHYSTYPE_FLUID|PX_PHYSTYPE_FORCEFIELD|PX_PHYSTYPE_KINEMATIC|PX_PHYSTYPE_METAL_CLOTH|PX_PHYSTYPE_RB_OVER|PX_PHYSTYPE_SBCLOTHING|PX_PHYSTYPE_SOFTBODY|PX_PHYSTYPE_STATIC|PX_PHYSTYPE_UNDEFINED|px_physXPanel|px_plugin_unitscale|px_plugin_unittype|px_plugin_version|PX_RAGDOLL_MESHFROM_BONE|PX_RAGDOLL_MESHFROM_SKIN|PX_RAGDOLL_MESHTYPE_CAPSULE|PX_RAGDOLL_MESHTYPE_CONVEX|PX_RAGDOLL_RBTYPE_DYNAMIC|PX_RAGDOLL_RBTYPE_KINEMATIC|px_rb_solveritertions|px_rbPanel_Advanced|px_rbPanel_basic|px_rbPanel_display|px_rbPanel_material|px_rbPanel_physicalMesh|px_sdk_bouncethresh|px_sdk_ccd_epsilon|px_sdk_ccd_motion_threshold|px_sdk_contactDistance|px_sdk_continuous_cd|px_sdk_dynamicfrictionscaling|px_sdk_enable_gravity|px_sdk_gravityDirection|px_sdk_gravityx|px_sdk_gravityy|px_sdk_gravityz|px_sdk_max_angvel|PX_SDK_MAXANGVEL|px_sdk_skinwidth|px_sdk_sleep_energy|px_sdk_sleeplinevel|px_sdk_staticfrictionscaling|px_sdk_sub_sim_steps|PX_SIMLOOP_REPEAT|PX_SIMLOOP_RESTART|px_tab_multiedit|px_version_converting|PxAtLeastMax2011|PxBakedNames|PxColorDynamicRB|PxColorKinematicRB|PxColorStaticRB|PxCreationPlaceAndSizeTool|pxCurrentNode|pxCurrentNodeType|pxCurrentShape|PxExporterParameters|PxIconCount|pxJoint|PxMatHullDynamicRB|PxMatHullKinematicRB|PxMatHullStaticRB|PxMaxValue|PxMeterToDisplayUnit|PxMeterToSystemUnit|PxModsToDelete|PxOpeningNewFile|PxRagNodePreDelete|pxScaleTools|PxSDK3Loaded|pxSelectionChangedDirty|pxSelectionTool|PxShowMainMenu|PxStringRolloutPanelEngine|PxTabIndexDisplay|PxTabIndexMultiEdit|PxTabIndexTool|PxTabIndexWorld|radiusManip|rApplyIKChain|RATE_DEGREE_TO_RAD|RATE_RAD_TO_DEGREE|rc2mxs|rcalc_floater|rcalc_rollout|rCaptureAnim|rCATRigMapping|rCollapseLayers|ReNameFloater|resetxformsini|rGizmos|Ribbon_Modeling_ManagedServices_SceneUtilities|RibbonColorIllumSelectType|RibbonDisplacementType|RibbonNURMSUpdateType|riggingini|rImportBIP|rImportBVH|rImportHTR|rLMR|rLoadFile|rltCollapseControllerSetup|rMergeAnim|rObjectMapping|rPoseMixer|rSaveFile|RTT_data|RTT_methods|sliderAAQualitySize|sliderManipulator|SoftSelectionManager|strRagdollHelperMesh|SymmetryMainModel|theFastSettingsRollout|thePcOps|thisFile|tvops|typeListAAQualityMethod|typeListAAQualityValues|typeListGIMethod|typeListGIQualityBaseValues|typeListGIQualityValues|typeListLabelPercents|typeListLightBounces|typeListShadingQualityValues|UIScheme_Bitmap_Init|unwrapUIdialog|uvwMappingHeightManip|uvwMappingLengthManip|uvwMappingPositionManip|uvwMappingWidthManip|ValidateExposureControlForLight_Dialog|VCanvas|VCBehaveRoll|VCBrushImageRoll|VCBrushImageSettingsRoll|VCBrushMenu|VCCustomBrushRollout|VCLayers|VCMainFloater|VCMapMenu|VCOptionsRoll|VCPaintRoll|VCPressureRoll|VCRandomizeRoll|VCSettingsRoll|VCSetup|VCview2D|VFB_methods|VFB_Rollout_Bottom|VFB_Rollout_TopLeft|VFB_Rollout_TopRight|vray2sidedmtl_params|vrayLLister|vrayLListerYOffset|vraymtl_params|vrayRTImageComplete|vraySceneConverterRollout|vrayVismatConverterRollout|vrscene_exportPath|vrscene_lastMaxFileName|vrscene_rendPath|vrscene_sceneFile|vrsceneExpInfo|vrsceneExportRendererSelectRollout|vrsceneExportRollout|Vsp_Gantry|Vsp_GantryNZ|Vsp_GantrySA|Vsp_Lamp|Vsp_Sign|Vsp_Signal|Vsp_Symb|Vsp_Tree|walkAssist)\\b(?=\\s|;|$)',
          name: 'support.variable.maxscript'
        },
        {
          match:
            '(?i)\\b(assetUser|black|blue|brown|CrashOnSaveCustAttrib|dontCollect|e|emptyVal|false|gray|green|HKEY_CLASSES_ROOT|HKEY_CURRENT_CONFIG|HKEY_CURRENT_USER|HKEY_LOCAL_MACHINE|HKEY_PERFORMANCE_DATA|HKEY_USERS|mesh|ok|orange|pi|red|true|undefined|unsupplied|white|x_axis|y_axis|yellow|z_axis)\\b(?=\\s|;|$)',
          name: 'support.variable.core.maxscript'
        },
        {
          match:
            '(?i)\\b(activegrid|activeMeditSlot|ambientColor|ambientColorController|animationRange|animButtonEnabled|animButtonState|backgroundColor|backgroundColorController|backgroundImageFileName|currentTime|displayGamma|displaySafeFrames|editorFont|editorFontSize|editorShowPath|editorTabWidth|enableAccelerators|environmentMap|escapeEnable|fileInGamma|fileOutGamma|flyOffTime|frameRate|globalTracks|hardwareLockID|heapFree|heapSize|hotspotAngleSeparation|inputTextColor|lightLevel|lightLevelController|lightTintColor|lightTintColorController|listener|localPreRendScript|localTime|lyricDirectory|macroRecorder|manipulateMode|maxFileName|maxFilePath|messageTextColor|numAtmospherics|numEffects|numSubObjectLevels|outputTextColor|playActiveOnly|playbackLoop|postRendScript|preRendScript|productAppID|realTimePlayback|rendAtmosphere|rendCamNode|rendColorCheck|rendDither256|rendDitherTrue|rendEnd|renderDisplacements|RenderEffects|renderer|renderHeight|renderPixelAspect|renderPresetMRUList|renderWidth|rendFieldOrder|rendFieldRender|rendFileNumberBase|rendForce2Side|rendHidden|rendImageAspectRatio|rendImgSeqType|rendLockImageAspectRatio|rendLockPixelAspectRatio|rendMultiThread|rendNThFrame|rendNThSerial|rendNTSC_PAL|rendOutputFilename|rendPickupFrames|rendPixelAspectRatio|rendSaveFile|rendShowVFB|rendSimplifyAreaLights|rendStart|rendSuperBlack|rendSuperBlackThresh|rendTimeType|rendUseActiveView|rendUseDevice|rendUseImgSeq|rendUseNet|rendVidCorrectMethod|rendViewID|rendViewIndex|rootNode|rootScene|scriptsPath|showEndResult|skipRenderedFrames|sliderTime|stackLimit|subObjectLevel|superclasses|ticksPerFrame|timeDisplayMode|trackViewNodes|useEnvironmentMap|usePostRendScript|usePreRendScript|videoPostTracks)\\b(?=\\s|;|$)',
          name: 'support.variable.system.maxscript'
        }
      ]
    }
  },
  scopeName: 'source.maxscript'
}

export default grammar
