/**
 * Create a `StarryNight` that can highlight things based on the given
 * `grammars`.
 * This is async to facilitate async loading and registering, which is currently
 * only used for WASM.
 *
 * @param {Array<Grammar>} grammars
 *   Grammars to support.
 */
export function createStarryNight(grammars: Array<Grammar>): Promise<{
  flagToScope: (flag: string) => string | undefined
  scopes: () => Array<string>
  register: (grammars: Array<Grammar>) => Promise<void>
  highlight: (value: string, scope: string) => Root
}>
export type Root = import('hast').Root
export type Grammar = GrammarFields & PatternFields
export type Rule =
  | RuleInclude
  | RuleName
  | RulePatterns
  | RuleDefinitionBeginEnd
  | RuleDefinitionBeginWhile
  | RuleDefinitionMatch
  | RuleDefinitionCaptures
export type RuleInclude = RuleIncludeFields
export type RuleName = RuleNameFields
export type RulePatterns = PatternFields
export type RuleDefinitionBeginEnd = RuleDefinitionSharedFields &
  RuleDefinitionBeginEndFields
export type RuleDefinitionBeginWhile = RuleDefinitionSharedFields &
  RuleDefinitionBeginWhileFields
export type RuleDefinitionMatch = RuleDefinitionSharedFields &
  RuleDefinitionMatchFields
export type RuleDefinitionCaptures = Omit<
  RuleDefinitionSharedFields,
  'captures'
> &
  RuleDefinitionCapturesFields
export type GrammarFields = {
  scopeName: string
  names: Array<string>
  extensions: Array<string>
}
export type PatternFields = {
  patterns: Array<Rule>
  repository?: Record<string, Rule> | undefined
  injections?: Record<string, Rule> | undefined
}
export type RuleNameFields = {
  name: string
}
export type RuleIncludeFields = {
  include: string
}
export type RuleDefinitionSharedFields = {
  name?: string | undefined
  contentName?: string | undefined
  captures?: Captures | undefined
  beginCaptures?: Captures | undefined
  endCaptures?: Captures | undefined
  applyEndPatternLast?: boolean | undefined
}
export type RuleDefinitionBeginEndFields = {
  begin: string
  end: string
}
export type RuleDefinitionBeginWhileFields = {
  begin: string
  while: string
}
export type RuleDefinitionMatchFields = {
  match: string
}
export type RuleDefinitionCapturesFields = {
  captures: Captures
}
export type Captures = Record<number | string, Rule>
